# -*- coding: utf-8 -*-
from zope.testbrowser.browser import HTTPError
import pytest


def test_address__menu__1(person_data, browser):
    """The entries in the add menu have a specific sort order."""
    browser.login('editor')
    browser.open(browser.PERSON_EDIT_URL)
    assert ['postal address',
            'phone number',
            'e-mail address',
            'home page',
            'file'] == browser.etree.xpath(
        '//ul[@id="add-menu-content"]/li/a/span/text()')


def test_address__AddPhoneNumberForm__1(person_data, browser):
    """`AddPhoneNumberForm` allows to add another phone number."""
    browser.login('editor')
    browser.open(browser.PERSON_EDIT_URL)
    browser.getLink('phone number').click()
    browser.getControl('number').value = '+4901761234567'
    browser.getControl('Add').click()
    assert '"+4901761234567" added.' == browser.message
    # After adding a new phone number, the person edit form is displayed
    # where the new phone number is shown:
    assert '+4901761234567' == browser.getControl('number', index=2).value
    # It is now possible to change this phone number in the person edit form:
    browser.getControl('number', index=2).value = '+4901767654321'
    browser.getControl('Apply').click()
    'Data successfully updated.' == browser.message
    browser.open(browser.PERSON_EDIT_URL)
    assert '+4901767654321' == browser.getControl('number', index=2).value


def test_address__AddPostalAddressForm__1(person_data, browser):
    """`AddPostalAddressForm` allows to add another postal address."""
    browser.login('editor')
    browser.open(browser.PERSON_EDIT_URL)
    browser.getLink('postal address').click()
    browser.getControl('address prefix').value = 'ABC-Computer'
    browser.getControl('street').value = 'Forsterstraße 302a'
    browser.getControl('city').value = 'Erfurt'
    browser.getControl('zip').value = '98344'
    browser.getControl('Add').click()
    assert (
        u'"ABC-Computer, Forsterstraße 302a, 98344, Erfurt, Germany" added.' ==
        browser.message)
    # After adding a new postal address, the person edit form is displayed
    # where the new postal address is shown:
    assert 'ABC-Computer' == browser.getControl(
        'address prefix', index=2).value
    assert 'Forsterstraße 302a' == browser.getControl('street', index=1).value
    assert 'Erfurt' == browser.getControl('city', index=2).value
    # It is now possible to change this postal address in the person edit form:
    browser.getControl('address prefix', index=2).value = 'RST-Software'
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    browser.open(browser.PERSON_EDIT_URL)
    assert 'RST-Software' == browser.getControl(
        'address prefix', index=2).value


def test_address__AddEMailAddressForm__1(person_data, browser):
    """`AddEMailAddressForm` allows to add another e-mail address."""
    browser.login('editor')
    browser.open(browser.PERSON_EDIT_URL)
    browser.getLink('e-mail address').click()
    browser.getControl('e-mail address').value = 'pt@abc-computer.de'
    browser.getControl('Add').click()
    assert '"pt@abc-computer.de" added.' == browser.message
    # After adding a new e-mail address, the person edit form is displayed
    # where the new e-mail address is shown:
    assert 'pt@abc-computer.de' == browser.getControl(
        'e-mail address', index=3).value
    # It is now possible to change this e-mail address in the person edit form:
    browser.getControl('e-mail address', index=3).value = 'pt@rst.example.edu'
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    browser.open(browser.PERSON_EDIT_URL)
    assert 'pt@rst.example.edu' == browser.getControl(
        'e-mail address', index=3).value


def test_address__AddHomePageAddressForm__1(person_data, browser):
    """`AddHomePageAddressForm` allows to add another home page address."""
    browser.login('editor')
    browser.open(browser.PERSON_EDIT_URL)
    browser.getLink('home page').click()
    browser.getControl('URL').value = 'http://www.abc-computer.de'
    browser.getControl('Add').click()
    assert '"http://www.abc-computer.de" added.' == browser.message
    # After adding a new home page address, the person edit form is
    # displayed where the new home page address is shown:
    assert 'http://www.abc-computer.de' == browser.getControl(
        'URL', index=2).value
    # It is now possible to change this home page address in the person edit
    # form:
    browser.getControl('URL', index=2).value = 'http://www.rst.example.edu'
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    browser.open(browser.PERSON_EDIT_URL)
    assert 'http://www.rst.example.edu' == browser.getControl(
        'URL', index=1).value


def test_address__DeletePostalAddressForm__1(person_data, browser):
    """`DeletePostalAddressForm` allows to delete a home page address."""
    browser.login('editor')
    browser.open(browser.PERSON_DELETE_ENTRY_URL)
    browser.getControl('RST-Software').click()
    browser.getControl('Delete entry').click()
    assert browser.POSTAL_ADDRESS_DELETE_URL == browser.url
    browser.getControl('Yes').click()
    assert (u'"RST-Software, Forsterstra\xdfe 302a, 98344, Erfurt, Germany" '
            u'deleted.' == browser.message)
    assert browser.PERSON_EDIT_URL
    assert (['c/o Mama, Demoweg 23, 88888, Testhausen, Austria'] ==
            browser.getControl('main postal address').displayOptions)


def test_address__DeletePostalAddressForm__2(person_data, browser):
    """`DeletePostalAddressForm` can be canceled."""
    browser.login('editor')
    browser.open(browser.PERSON_DELETE_ENTRY_URL)
    browser.getControl('RST-Software').click()
    browser.getControl('Delete entry').click()
    browser.getControl('No, cancel').click()
    assert 'Deletion canceled.' == browser.message
    assert browser.PERSON_EDIT_URL == browser.url


def test_address__DeletePostalAddressForm__3(person_data, browser):
    """Deleting the main address creates a new, empty address."""
    browser.login('editor')
    browser.open(browser.PERSON_DELETE_ENTRY_URL)
    browser.getControl('Demoweg').click()  # deselects default selection
    browser.getControl('Demoweg').click()
    browser.getControl('Delete entry').click()
    browser.getControl('Yes').click()
    assert ('"c/o Mama, Demoweg 23, 88888, Testhausen, Austria" deleted.' ==
            browser.message)
    assert browser.PERSON_EDIT_URL == browser.url
    assert (['Germany'] ==
            browser.getControl('main postal address').displayValue)
    assert (
        ['RST-Software, Forsterstra\xc3\x9fe 302a, 98344, Erfurt, Germany',
         'Germany'] ==
        browser.getControl('main postal address').displayOptions)


def test_address__DeletePostalAddressForm__4(person_data, browser):
    """`DeletePostalAddressForm` cannot be accessed by a visitor."""
    browser.login('visitor')
    with pytest.raises(HTTPError) as err:
        browser.open(browser.POSTAL_ADDRESS_DELETE_URL)
    assert 'HTTP Error 403: Forbidden' == str(err.value)


def test_address__DeletePhoneNumberForm__1(person_data, browser):
    """`DeletePhoneNumberForm` allows to delete a phone number."""
    browser.login('editor')
    browser.open(browser.PERSON_DELETE_ENTRY_URL)
    browser.getControl('+4901761234567').click()
    browser.getControl('Delete entry').click()
    assert browser.PHONE_NUMBER_DELETE_URL == browser.url
    browser.getControl('Yes').click()
    assert ('"+4901761234567" deleted.' == browser.message)
    assert browser.PERSON_EDIT_URL
    assert (['+4901767654321'] ==
            browser.getControl('main phone number').displayOptions)


def test_address__DeletePhoneNumberForm__2(person_data, browser):
    """`DeletePhoneNumberForm` cannot be accessed by a visitor."""
    browser.login('visitor')
    with pytest.raises(HTTPError) as err:
        browser.open(browser.PHONE_NUMBER_DELETE_URL)
    assert 'HTTP Error 403: Forbidden' == str(err.value)


def test_address__DeleteEMailAddressForm__1(person_data, browser):
    """`DeleteEMailAddressForm` allows to delete an email address."""
    browser.login('editor')
    browser.open(browser.PERSON_DELETE_ENTRY_URL)
    browser.getControl('pt@rst.example.edu').click()
    browser.getControl('Delete entry').click()
    assert browser.EMAIL_ADDRESS_DELETE_URL == browser.url
    browser.getControl('Yes').click()
    assert ('"pt@rst.example.edu" deleted.' == browser.message)
    assert browser.PERSON_EDIT_URL
    assert (['petra@example.com'] ==
            browser.getControl('main e-mail address').displayOptions)


def test_address__DeleteEMailAddressForm__2(person_data, browser):
    """`DeleteEMailAddressForm` cannot be accessed by a visitor."""
    browser.login('visitor')
    with pytest.raises(HTTPError) as err:
        browser.open(browser.EMAIL_ADDRESS_DELETE_URL)
    assert 'HTTP Error 403: Forbidden' == str(err.value)


def test_address__DeleteHomePageAddressForm__1(person_data, browser):
    """`DeleteHomePageAddressForm` allows to delete a home page address."""
    browser.login('editor')
    browser.open(browser.PERSON_DELETE_ENTRY_URL)
    browser.getControl('http://www.rst.example.edu').click()
    browser.getControl('Delete entry').click()
    assert browser.HOMEPAGE_ADDRESS_DELETE_URL == browser.url
    browser.getControl('Yes').click()
    assert ('"http://www.rst.example.edu" deleted.' == browser.message)
    assert browser.PERSON_EDIT_URL
    assert (['http://petra.example.com'] ==
            browser.getControl('main home page address').displayOptions)


def test_address__DeleteHomePageAddressForm__2(person_data, browser):
    """`DeleteHomePageAddressForm` cannot be accessed by a visitor."""
    browser.login('visitor')
    with pytest.raises(HTTPError) as err:
        browser.open(browser.HOMEPAGE_ADDRESS_DELETE_URL)
    assert 'HTTP Error 403: Forbidden' == str(err.value)



import zope.interface


class IExporter(zope.interface.Interface):
    """Exporting facility."""

    file_extension = zope.interface.Attribute(
        u'Extension (without the leading dot!) to be set on export file name.')
    mime_type = zope.interface.Attribute(u'Mime-type of the export file.')

    def export():
        """Export to a file.

        Returns a file or file-like-object.

        """



from icemac.addressbook.i18n import _
import grokcore.component
import icemac.addressbook.browser.interfaces


@grokcore.component.adapter(None, ZeroDivisionError)
@grokcore.component.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def zero_division_error(field, exc):
    return _('Division by zero')


@grokcore.component.adapter(None, Exception)
@grokcore.component.implementer(
    icemac.addressbook.browser.interfaces.IErrorMessage)
def ordinary_exception(field, exc):
    return _('Unexpected error occurred: ${klass}: ${text}',
             mapping=dict(klass=exc.__class__.__name__, text=exc))



# -*- coding: utf-8 -*-
from ..base import update_persons, clean_update_data_session
from icemac.addressbook.person import Person, person_entity
import mock
import zope.i18n
import zope.publisher.browser


def test_base__update_persons__1(zcmlS):
    """It updates only the specified field."""
    person1 = Person()
    person1.notes = u'p1'
    person2 = Person()
    person2.notes = u'p2'
    update_persons((person1, person2), person_entity,
                   person_entity.getRawField('notes'), 'append', u'bar')
    assert 'p1bar' == person1.notes
    assert 'p2bar' == person2.notes
    # no other attributes are in vars as they are not changed (only the
    # private container attributes are additionally here).
    assert set(['notes', '_BTreeContainer__len', '_SampleContainer__data',
                '__annotations__']) == set(vars(person1).keys())


def test_base__update_persons__2(zcmlS):
    """It returns a ZeroDivision message if dividing by zero."""
    person1 = Person()
    person1.__name__ = 'person1'
    call = ('icemac.addressbook.browser.search.result.handler.update.'
            'operators.NoneAppend.__call__')
    with mock.patch(call, side_effect=ZeroDivisionError):
        result = update_persons(
            [person1], person_entity,
            person_entity.getRawField('notes'), 'append', u'bar')
    assert {'person1': u'Division by zero'} == result


def test_base__update_persons__3(zcmlS):
    """It returns an Exception if another exception occurred."""
    call = ('icemac.addressbook.browser.search.result.handler.update.'
            'operators.NoneAppend.__call__')
    person1 = Person()
    person1.__name__ = 'person1'
    with mock.patch(call, side_effect=IOError('file not found')):
        result = update_persons(
            [person1], person_entity,
            person_entity.getRawField('notes'), 'append', u'bar')
    errors = {key: zope.i18n.translate(val) for key, val in result.items()}
    assert (
        {'person1': u'Unexpected error occurred: IOError: file not found'} ==
        errors)


def test_base__clean_update_data_session__1(address_book):
    """It does not break if `UPDATE_SESSION_KEY` is not in the session."""
    request = zope.publisher.browser.TestRequest()
    clean_update_data_session(request)



from icemac.addressbook.i18n import _
from icemac.addressbook.principals.sources import role_source
import icemac.addressbook.browser.base
import icemac.addressbook.browser.menus.menu
import icemac.addressbook.browser.metadata
import icemac.addressbook.browser.table
import icemac.addressbook.interfaces
import icemac.addressbook.principals.interfaces
import icemac.addressbook.principals.principals
import transaction
import z3c.form.button
import z3c.form.field
import z3c.form.validator
import z3c.form.widget
import zope.interface
import zope.pluggableauth.plugins.principalfolder
import zope.schema
import zope.schema.interfaces
import zope.security


class LastLoginColumn(icemac.addressbook.browser.table.DateTimeColumn):

    header = _(u'last login')
    formatterLength = u'short'

    def getRawValue(self, item):
        value = item.last_login
        if value:
            tz = icemac.addressbook.preferences.utils.get_time_zone()
            value = tz.normalize(value)
        return value


class Overview(icemac.addressbook.browser.table.PageletTable):

    no_rows_message = _(
        u'No users defined yet or you are not allowed to access any.')

    def setUpColumns(self):
        return [
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TitleLinkColumn,
                'user', weight=1),
            z3c.table.column.addColumn(
                self, z3c.table.column.GetAttrColumn, 'login', weight=2,
                header=_(u'login name'), attrName='login'),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.SourceColumn, 'roles',
                header=_(u'roles'), attrName='roles', source=role_source,
                weight=3),
            z3c.table.column.addColumn(
                self, LastLoginColumn, 'last_login', weight=4),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TruncatedContentColumn,
                'notes', weight=5,
                header=_(u'notes'), attrName='description', length=50),
        ]

    @property
    def values(self):
        for principal in self.context.values():
            if zope.security.canAccess(principal, 'login'):
                yield principal


class PersonFieldDataManager(z3c.form.datamanager.AttributeField):
    """Person is a readonly field which should be written once."""

    zope.component.adapts(
        icemac.addressbook.principals.interfaces.IPrincipal,
        zope.schema.interfaces.IChoice)

    def set(self, value):
        # `person` is a read-only field, so we cannot use the super call:
        self.context.person = value


class AddForm(icemac.addressbook.browser.base.BaseAddForm):

    label = _(u'Add new user')
    class_ = icemac.addressbook.principals.principals.Principal
    interface = icemac.addressbook.principals.interfaces.IPrincipal
    next_url = 'parent'

    @property
    def fields(self):
        return (
            z3c.form.field.Fields(
                icemac.addressbook.principals.interfaces.IPrincipal).select(
                'person') +
            z3c.form.field.Fields(
                icemac.addressbook.principals.interfaces.IPasswordFields) +
            z3c.form.field.Fields(
                icemac.addressbook.principals.interfaces.IRoles) +
            z3c.form.field.Fields(
                *icemac.addressbook.interfaces.IEntity(
                    self.interface).getFieldValues()).omit(
                'login', 'person')
        )

    def add(self, obj):
        try:
            return super(AddForm, self).add(obj)
        except zope.container.interfaces.DuplicateIDError, e:
            transaction.doom()
            raise z3c.form.interfaces.ActionExecutionError(
                zope.interface.Invalid(_(e.args[0])))


# in the Add form the password fields are required ...
password_required = z3c.form.widget.StaticWidgetAttribute(
    True, context=None, request=None, view=AddForm, field=zope.schema.Password,
    widget=None)

# ... but not in all other forms
password_not_required = z3c.form.widget.StaticWidgetAttribute(
    False, context=icemac.addressbook.principals.interfaces.IPrincipal,
    request=None, view=None, field=zope.schema.Password, widget=None)


class EditForm(icemac.addressbook.browser.base.GroupEditForm):

    label = _(u'Edit user')
    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)
    interface = icemac.addressbook.principals.interfaces.IPrincipal
    next_url = 'parent'

    @property
    def fields(self):
        return (
            z3c.form.field.Fields(
                icemac.addressbook.principals.interfaces.IPrincipal).select(
                'person', 'login') +
            z3c.form.field.Fields(
                icemac.addressbook.principals.interfaces.IPasswordFields) +
            z3c.form.field.Fields(
                icemac.addressbook.principals.interfaces.IRoles) +
            z3c.form.field.Fields(
                *icemac.addressbook.interfaces.IEntity(
                    self.interface).getFieldValues()).omit(
                'person', 'login'))

    @z3c.form.button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll loose it.
        super(EditForm, self).handleApply(self, action)

    @z3c.form.button.buttonAndHandler(
        _(u'Delete user'), name='delete_user',
        condition=icemac.addressbook.browser.base.can_access(
            'delete_user.html'))
    def handleDeleteUser(self, action):
        self.redirect_to_next_url('object', 'delete_user.html')

    def applyChanges(self, data):
        current_principal_id = (
            self.request.interaction.participations[0].principal.id)
        current_login_name = None
        auth = zope.component.getUtility(
            zope.authentication.interfaces.IAuthentication)
        current_login_names = [
            getattr(plugin.principalInfo(current_principal_id), 'login', None)
            for key, plugin in auth.getAuthenticatorPlugins()]
        assert len(current_login_names) == 1
        current_login_name = current_login_names[0]

        old_login_name = self.context.login
        editing_own_data = (old_login_name == current_login_name)
        try:
            changes = super(EditForm, self).applyChanges(data)
        except ValueError, e:
            transaction.doom()
            raise z3c.form.interfaces.ActionExecutionError(
                zope.interface.Invalid(_(e.args[0])))

        if not editing_own_data:
            # User is not editing his own data principal's data:
            return changes

        changed_field_names = []
        changed_field_names.extend(
            changes.get(
                icemac.addressbook.principals.interfaces.IPrincipal, []))
        changed_field_names.extend(
            changes.get(
                icemac.addressbook.principals.interfaces.IPasswordFields, []))
        if 'login' in changed_field_names:
            self.send_flash(_('You changed the login name, please re-login.'))
        if ('password' in changed_field_names and
                'password_repetition' in changed_field_names and
                data['password']):
            self.send_flash(_('You changed the password, please re-login.'))
        return changes


class EditForm_password_Validator(z3c.form.validator.SimpleFieldValidator):
    """Validator for data field in add form."""

    zope.component.adapts(
        icemac.addressbook.principals.interfaces.IPrincipal, None, EditForm,
        zope.schema.Password, None)

    def validate(self, value):
        if not value:
            # use a dummy value for validation, so passwords needn't
            # be entered everytime the edit form is saved
            value = u'asdfasdf'
        return super(EditForm_password_Validator, self).validate(value)


class DeleteUserForm(icemac.addressbook.browser.base.BaseDeleteForm):
    label = _(u'Do you really want to delete this user?')
    interface = icemac.addressbook.principals.interfaces.IPrincipal
    field_names = ('person', 'login')


principal_views = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    zope.pluggableauth.plugins.principalfolder.IInternalPrincipalContainer,
    icemac.addressbook.principals.interfaces.IPrincipal)



# -*- coding: utf-8 -*-
import datetime
import decimal
from icemac.addressbook.interfaces import IEntityOrder, IEntity, IPhoneNumber
from icemac.addressbook.interfaces import IPerson, IPostalAddress
import icemac.addressbook.export.interfaces
import icemac.addressbook.export.xls.simple
import pytest
import xlrd
import zope.component
import zope.interface.verify


@pytest.fixture(scope='function')
def user_defined_fields_data(
        address_book, FieldFactory, FullPersonFactory, PhoneNumberFactory,
        PostalAddressFactory):
    """Fixture setting up the test data for user defined fields tests."""
    FieldFactory(address_book, IPerson, u'Bool', u'photo permission?')
    FieldFactory(address_book, IPerson, u'Datetime', u'last seen')
    FieldFactory(address_book, IPostalAddress, u'Choice', u'state',
                 values=[u'Sachsen', u'Sachsen-Anhalt', u'Brandenburg'])
    FieldFactory(address_book, IPostalAddress, u'Int', u'number of letters')
    FieldFactory(address_book, IPhoneNumber, u'Decimal', u'cost per minute')
    FieldFactory(address_book, IPhoneNumber, u'Text', u'mail box text')

    liebig = FullPersonFactory(
        address_book, u'Liebig',
        **{'person__first_name': u'B.',
           'person__Field-1': True,
           'person__Field-2': datetime.datetime(2009, 11, 1, 14, 1),
           'postal__city': u'Testhausen',
           'postal__Field-3': u'Sachsen-Anhalt',
           'postal__Field-4': 3,
           'phone__number': u'01234-5678-90',
           'phone__Field-5': decimal.Decimal('1.421'),
           'phone__Field-6': 'I am not here, leave a message, beep.'})
    howitz = FullPersonFactory(
        address_book, u'Howitz',
        **{'person__Field-1': False,
           'postal__city': u'Hettstedt',
           'postal__Field-3': u'Brandenburg'})

    # Let's create some additional phone numbers and postal addresses which
    # do not show up in defaults export but in complete export:
    PhoneNumberFactory(liebig, **{'Field-5': decimal.Decimal('1e2')})
    PhoneNumberFactory(liebig, **{'number': u'+49-3453-23434',
                                  'Field-5': decimal.Decimal('28')})
    PostalAddressFactory(howitz, **{'zip': u'00001', 'Field-4': 123456})
    return liebig, howitz


def test_simple__XLSExport__1():
    """`XLSExport` conforms to `IExporter`."""
    assert zope.interface.verify.verifyObject(
        icemac.addressbook.export.interfaces.IExporter,
        icemac.addressbook.export.xls.simple.XLSExport([], None))


def test_simple__DefaultsExport__1():
    """`DefaultsExport` conforms to `IExporter`."""
    assert zope.interface.verify.verifyObject(
        icemac.addressbook.export.interfaces.IExporter,
        icemac.addressbook.export.xls.simple.DefaultsExport([], None))


def test_simple__DefaultsExport__2(person_data, PhoneNumberFactory):
    """Export personal data, *main* addresses and numbers to an XLS file."""
    person = person_data['Person']
    # Additional phone numbers which does not show up in the export.
    PhoneNumberFactory(person, number=u'017612345678')
    export = icemac.addressbook.export.xls.simple.DefaultsExport(
        [person]).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    assert [u'Address book - Export'] == xls_workbook.sheet_names()
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (3, 13) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert [
        [u'person',
         '',
         '',
         '',
         '',
         u'postal address',
         '',
         '',
         '',
         '',
         u'phone number',
         u'e-mail address',
         u'home page address'],
        [u'first name',
         u'last name',
         u'birth date',
         u'keywords',
         u'notes',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'number',
         u'e-mail address',
         u'URL'],
        [u'Petra',
         u'Tester',
         '',
         u'family, church',
         '',
         u'c/o Mama',
         u'Demoweg 23',
         u'Testhausen',
         u'88888',
         u'Austria',
         u'+4901767654321',
         u'petra@example.com',
         u'http://petra.example.com']] == [work_sheet_0.row_values(rx)
                                           for rx in range(work_sheet_0.nrows)]


def test_simple__DefaultsExport__3(address_book):
    """`DefaultsExport` returns a quite empty XLS file if nothing to export."""
    export = icemac.addressbook.export.xls.simple.DefaultsExport([]).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (2, 13) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert [
        [u'person',
         '',
         '',
         '',
         '',
         u'postal address',
         '',
         '',
         '',
         '',
         u'phone number',
         u'e-mail address',
         u'home page address'],
        [u'first name',
         u'last name',
         u'birth date',
         u'keywords',
         u'notes',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'number',
         u'e-mail address',
         u'URL']] == [work_sheet_0.row_values(rx)
                      for rx in range(work_sheet_0.nrows)]


def test_simple__DefaultsExport__4(address_book):
    """`DefaultsExport` respects the user defined entity sort order."""
    entity_order = zope.component.getUtility(IEntityOrder)
    entity_order.up(IEntity(IPhoneNumber), 1)

    export = icemac.addressbook.export.xls.simple.DefaultsExport([]).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (2, 13) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert [
        [u'person',
         '',
         '',
         '',
         '',
         u'phone number',
         u'postal address',
         '',
         '',
         '',
         '',
         u'e-mail address',
         u'home page address'],
        [u'first name',
         u'last name',
         u'birth date',
         u'keywords',
         u'notes',
         u'number',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'e-mail address',
         u'URL']] == [work_sheet_0.row_values(rx)
                      for rx in range(work_sheet_0.nrows)]


def test_simple__DefaultsExport__5(address_book):
    """`DefaultsExport` respects the user defined field sort order."""
    person_entity = IEntity(IPerson)
    person_entity.setFieldOrder(('last_name', 'first_name'))

    export = icemac.addressbook.export.xls.simple.DefaultsExport([]).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (2, 13) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert [
        [u'person',
         '',
         '',
         '',
         '',
         u'postal address',
         '',
         '',
         '',
         '',
         u'phone number',
         u'e-mail address',
         u'home page address'],
        [u'last name',
         u'first name',
         u'birth date',
         u'keywords',
         u'notes',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'number',
         u'e-mail address',
         u'URL']] == [work_sheet_0.row_values(rx)
                      for rx in range(work_sheet_0.nrows)]


def test_simple__DefaultsExport__6(user_defined_fields_data):
    """`DefaultsExport` exports user defined fields alike other fields."""
    export = icemac.addressbook.export.xls.simple.DefaultsExport(
        user_defined_fields_data).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (4, 19) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert [
        [u'person',
         '',
         '',
         '',
         '',
         '',
         '',
         u'postal address',
         '',
         '',
         '',
         '',
         '',
         '',
         u'phone number',
         '',
         '',
         u'e-mail address',
         u'home page address'],
        [u'first name',
         u'last name',
         u'birth date',
         u'keywords',
         u'notes',
         u'photo permission?',
         u'last seen',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'number of letters',
         u'state',
         u'number',
         u'cost per minute',
         u'mail box text',
         u'e-mail address',
         u'URL'],
        [u'B.',
         u'Liebig',
         '',
         '',
         '',
         1,
         40118.584027777775,
         '',
         '',
         u'Testhausen',
         '',
         u'Germany',
         3.0,
         u'Sachsen-Anhalt',
         u'01234-5678-90',
         1.421,
         u'I am not here, leave a message, beep.',
         '',
         ''],
        ['',
         u'Howitz',
         '',
         '',
         '',
         0,
         '',
         '',
         '',
         u'Hettstedt',
         '',
         u'Germany',
         '',
         u'Brandenburg',
         '',
         '',
         '',
         '',
         '']] == [work_sheet_0.row_values(rx)
                  for rx in range(work_sheet_0.nrows)]


def test_simple__CompleteExport__1():
    """`CompleteExport` conforms to `IExporter`."""
    assert zope.interface.verify.verifyObject(
        icemac.addressbook.export.interfaces.IExporter,
        icemac.addressbook.export.xls.simple.CompleteExport([], None))


def test_simple__CompleteExport__2(person_data):
    """Export personal data, all addresses and numbers to an XLS file."""
    person = person_data['Person']
    export = icemac.addressbook.export.xls.simple.CompleteExport(
        [person]).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    assert [u'Address book - Export'] == xls_workbook.sheet_names()
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (3, 21) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert ([
        [u'person',
         '',
         '',
         '',
         '',
         u'main postal address',
         '',
         '',
         '',
         '',
         u'other postal address',
         '',
         '',
         '',
         '',
         u'main phone number',
         u'other phone number',
         u'main e-mail address',
         u'other e-mail address',
         u'main home page address',
         u'other home page address'],
        [u'first name',
         u'last name',
         u'birth date',
         u'keywords',
         u'notes',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'number',
         u'number',
         u'e-mail address',
         u'e-mail address',
         u'URL',
         u'URL'],
        [u'Petra',
         u'Tester',
         '',
         u'family, church',
         '',
         u'c/o Mama',
         u'Demoweg 23',
         u'Testhausen',
         u'88888',
         u'Austria',
         u'RST-Software',
         u'Forsterstraße 302a',
         u'Erfurt',
         u'98344',
         u'Germany',
         u'+4901767654321',
         u'+4901761234567',
         u'petra@example.com',
         u'pt@rst.example.edu',
         u'http://petra.example.com',
         u'http://www.rst.example.edu']] ==
        [work_sheet_0.row_values(rx)
         for rx in range(work_sheet_0.nrows)])


def test_simple__CompleteExport__3(address_book):
    """`CompleteExport` returns a quite empty XLS file if nothing to export."""
    export = icemac.addressbook.export.xls.simple.CompleteExport([]).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (2, 5) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert ([
        [u'person', '', '', '', ''],
        [u'first name', u'last name', u'birth date', u'keywords', u'notes']] ==
        [work_sheet_0.row_values(rx)
         for rx in range(work_sheet_0.nrows)])


def test_simple__CompleteExport__4(
        address_book, PersonFactory, PhoneNumberFactory):
    """`CompleteExport` respects the user defined entity sort order."""
    person = PersonFactory(address_book, u'Tester')
    PhoneNumberFactory(person, number=u'+48-123-321', set_as_default=True)
    PhoneNumberFactory(person, number=u'+321-48-84', set_as_default=False)

    entity_order = zope.component.getUtility(IEntityOrder)
    phone_number_entity = IEntity(IPhoneNumber)
    entity_order.up(phone_number_entity, 1)

    export = icemac.addressbook.export.xls.simple.CompleteExport(
        [person]).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (3, 7) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert ([
        [u'person',
         '',
         '',
         '',
         '',
         u'main phone number',
         u'other phone number'],
        [u'first name',
         u'last name',
         u'birth date',
         u'keywords',
         u'notes',
         u'number',
         u'number'],
        ['',
         u'Tester',
         '',
         '',
         '',
         u'+48-123-321',
         u'+321-48-84']] == [work_sheet_0.row_values(rx)
                             for rx in range(work_sheet_0.nrows)])


def test_simple__CompleteExport__5(address_book, ):
    """`CompleteExport` respects the user defined field sort order."""
    person_entity = IEntity(IPerson)
    person_entity.setFieldOrder(('last_name', 'first_name'))

    export = icemac.addressbook.export.xls.simple.CompleteExport([]).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (2, 5) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert ([
        [u'person',
         '',
         '',
         '',
         ''],
        [u'last name',
         u'first name',
         u'birth date',
         u'keywords',
         u'notes']] == [work_sheet_0.row_values(rx)
                        for rx in range(work_sheet_0.nrows)])


def test_simple__CompleteExport__6(user_defined_fields_data):
    """`CompleteExport` exports user defined fields alike other fields."""
    export = icemac.addressbook.export.xls.simple.CompleteExport(
        user_defined_fields_data).export()
    xls_workbook = xlrd.open_workbook(file_contents=export)
    work_sheet_0 = xls_workbook.sheet_by_index(0)
    assert (4, 32) == (work_sheet_0.nrows, work_sheet_0.ncols)
    assert [
        [u'person',
         '',
         '',
         '',
         '',
         '',
         '',
         u'main postal address',
         '',
         '',
         '',
         '',
         '',
         '',
         u'other postal address',
         '',
         '',
         '',
         '',
         '',
         '',
         u'main phone number',
         '',
         '',
         u'other phone number',
         '',
         '',
         u'other phone number',
         '',
         '',
         u'main e-mail address',
         u'main home page address'],
        [u'first name',
         u'last name',
         u'birth date',
         u'keywords',
         u'notes',
         u'photo permission?',
         u'last seen',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'number of letters',
         u'state',
         u'address prefix',
         u'street',
         u'city',
         u'zip',
         u'country',
         u'number of letters',
         u'state',
         u'number',
         u'cost per minute',
         u'mail box text',
         u'number',
         u'cost per minute',
         u'mail box text',
         u'number',
         u'cost per minute',
         u'mail box text',
         u'e-mail address',
         u'URL'],
        [u'B.',
         u'Liebig',
         '',
         '',
         '',
         1,
         40118.584027777775,
         '',
         '',
         u'Testhausen',
         '',
         u'Germany',
         3.0,
         u'Sachsen-Anhalt',
         '',
         '',
         '',
         '',
         '',
         '',
         '',
         u'01234-5678-90',
         1.421,
         u'I am not here, leave a message, beep.',
         '',
         100.0,
         '',
         u'+49-3453-23434',
         28.0,
         '',
         '',
         ''],
        ['',
         u'Howitz',
         '',
         '',
         '',
         0,
         '',
         '',
         '',
         u'Hettstedt',
         '',
         u'Germany',
         '',
         u'Brandenburg',
         '',
         '',
         '',
         u'00001',
         u'Germany',
         123456.0,
         '',
         '',
         '',
         '',
         '',
         '',
         '',
         '',
         '',
         '',
         '',
         '']] == [work_sheet_0.row_values(rx)
                  for rx in range(work_sheet_0.nrows)]



from .interfaces import IMasterData, IMasterDataMenuItemOn
import grokcore.component as grok
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.menus.menu
import z3c.menu.ready2go.item
import z3c.menu.ready2go.manager
import zope.interface
import zope.viewlet.manager


class MasterDataMenuItem(z3c.menu.ready2go.item.SiteMenuItem):
    """Site menu item for the master data tab."""


class MasterDataMenuItemSelectedChecker(
        icemac.addressbook.browser.menus.menu.SubscriberSelectedChecker):
    """Selected checker for the master data menu item in the site menu."""

    grok.adapts(zope.interface.Interface,
                icemac.addressbook.browser.interfaces.IAddressBookLayer,
                zope.interface.Interface,
                icemac.addressbook.browser.menus.menu.MainMenu,
                MasterDataMenuItem)
    subscriber_interface = IMasterDataMenuItemOn


# Menu on the master date page displaying the different master data edit links:
MasterDataManager = zope.viewlet.manager.ViewletManager(
    'master-data', IMasterData, bases=(
        z3c.menu.ready2go.manager.MenuManager,))



def test_default__add__1i(address_book, browser):
    """The default of the batch size is 20."""
    browser.login('visitor')
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    browser.getLink('Preferences').click()
    assert browser.PREFS_URL == browser.url
    assert '20' == browser.getControl('batch size').value


def test_default__add__2i(address_book, browser):
    """The default sort direction is ascending."""
    browser.login('visitor')
    browser.open(browser.PREFS_URL)
    assert (['ascending (A-->Z)'] ==
            browser.getControl('sort direction').displayValue)


def test_default__add__3i(address_book, browser):
    """The default order by is person's last name."""
    browser.login('visitor')
    browser.open(browser.PREFS_URL)
    assert (['person -- last name'] ==
            browser.getControl('order by').displayValue)


def test_default__add__4i(address_book, browser):
    """By default two columns are displayed in the person lists."""
    browser.login('visitor')
    browser.open(browser.PREFS_URL)
    # Two columns are displayed by default. As the widget is a
    # JavaScript-In-Out-Widget, we have to access the second list here:
    assert (['person -- last name', 'person -- first name'] ==
            browser.getControl(name='form.widgets.columns.to').displayOptions)


def test_default__add__5i(address_book, browser):
    """The default time zone is UTC."""
    browser.login('visitor')
    browser.open(browser.PREFS_TIMEZONE_URL)
    assert ['UTC'] == browser.getControl('Time zone').displayValue



from icemac.addressbook.i18n import _
from .interfaces import IBirthDate
import icemac.addressbook.browser.base
import zope.component


class ExportList(icemac.addressbook.browser.base.BaseView):
    """List available export formats."""

    def exporters(self):
        """Iterable of exporters having enough data so export something."""
        # XXX: This has no API, the exporters should be subscription adapters
        #      which return None if they have not enough data to export
        #      something and a dict consting of title and URL otherwise.
        birthdate_data = zope.component.getMultiAdapter(
            (self.context, self.request), IBirthDate)
        if birthdate_data.icalendar_event is not None:
            yield dict(title=_('iCalendar export of birth date (.ics file)'),
                       url=self.url(self.context, 'iCalendar'))

    def back_url(self):
        return self.url(self.context)



from mock import patch


def test_iCalendar__1(search_data, browser):
    """Smoke testing that an iCal file gets exported."""
    browser.login('visitor')
    with patch('pkg_resources.get_distribution') as get_distribution:
        get_distribution().version = '1.2.3'
        browser.keyword_search('family', apply='iCal')
    assert ['BEGIN:VCALENDAR',
            'VERSION:2.0',
            'PRODID:-//icemac.addressbook//1.2.3//EN',
            'CALSCALE:GREGORIAN',
            'X-WR-CALNAME:birthdays of test addressbook',
            'BEGIN:VEVENT',
            'SUMMARY:Koch (*1952)',
            'DTSTART;VALUE=DATE:19520124',
            'DTEND;VALUE=DATE:19520125',
            'UID:http://localhost/ab/Person-2/@@iCalendar',
            'RRULE:FREQ=YEARLY',
            'END:VEVENT',
            'END:VCALENDAR',
            ''] == browser.contents.split('\r\n')



from icemac.addressbook.addressbook import AddressBook
from icemac.addressbook.addressbook import create_address_book_infrastructure
from icemac.addressbook.interfaces import IAddressBook, IKeywords, IEntities
from icemac.addressbook.interfaces import IOrderStorage, ENTITIES
import zope.authentication.interfaces
import zope.catalog.interfaces
import zope.component
import zope.interface.verify
import zope.intid.interfaces
import zope.location.interfaces
import zope.pluggableauth.interfaces


def test_addressbook__add_more_addressbook_infrastructure__1(
        address_book, PersonFactory):
    """`add_more_addressbook_infrastructure` installs an IntId utility."""
    # Persons added to the address book are added to the IntID utility of the
    # address book. (The int id changes with every run so we can only show,
    # that there is an int id registered for the person):
    person = PersonFactory(address_book, u'Kohn')
    intids = zope.component.getUtility(zope.intid.interfaces.IIntIds)
    assert None is not intids.queryId(person)
    assert isinstance(intids.queryId(person), int)
    assert person is intids.queryObject(intids.queryId(person))
    assert None is intids.queryId(object())


def test_addressbook__add_more_addressbook_infrastructure__2(
        address_book, PersonFactory):
    """`add_more_addressbook_infrastructure` creates an index for keywords."""
    # The keywords on persons are indexed.
    person = PersonFactory(
        address_book, u'Kohn', keywords=[u'church', u'friends'])
    # The index should now contain two words (the titles of the keywords.),
    # so we can search for them:
    catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
    assert 2 == catalog.get('keywords').wordCount.value
    results = catalog.searchResults(keywords={'any_of': ('friends', )})
    assert 1 == len(results)
    assert person is list(results)[0]


def test_addressbook__add_more_addressbook_infrastructure__3(
        address_book, PersonFactory):
    """`add_more_addressbook_infrastructure` creates an index for names."""
    # The first name and last name of the persons are indexed to be used in
    # quick search:
    person = PersonFactory(address_book, u'Kohn', first_name=u'Ernst')
    catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
    assert 1 == len(catalog.searchResults(name='K??n'))
    assert person is list(catalog.searchResults(name='K??n'))[0]
    assert 1 == len(catalog.searchResults(name='Erns*'))
    assert 0 == len(catalog.searchResults(name='Erns'))


def check_addressbook(assert_address_book):
    """Check the validity and completeness of the address book."""
    assert zope.location.interfaces.ISite.providedBy(
        assert_address_book.address_book)
    assert_address_book.has_attribute('keywords', IKeywords)
    assert_address_book.has_attribute(
        'principals', zope.pluggableauth.interfaces.IAuthenticatorPlugin,
        name=u'icemac.addressbook.principals')
    assert_address_book.has_attribute('entities', IEntities)
    assert_address_book.has_attribute('orders', IOrderStorage)
    assert_address_book.has_local_utility(zope.intid.interfaces.IIntIds)
    assert_address_book.has_local_utility(zope.catalog.interfaces.ICatalog)
    assert_address_book.has_local_utility(
        zope.authentication.interfaces.IAuthentication)


def test_addressbook__create_address_book_infrastructure__1(
        assert_address_book):
    """The address_book created within the fixture is valid."""
    check_addressbook(assert_address_book)


def test_addressbook__create_address_book_infrastructure__2(
        assert_address_book):
    """Calling `create_address_book_infrastructure` again does not break."""
    create_address_book_infrastructure(assert_address_book.address_book)
    check_addressbook(assert_address_book)


def test_addressbook__create_address_book_infrastructure__3(address_book):
    """`create_address_book_infrastructure` creates default entity order."""
    assert ([
        'IcemacAddressbookAddressbookAddressbook',
        'IcemacAddressbookPersonPerson',
        'IcemacAddressbookPersonPersondefaults',
        'IcemacAddressbookAddressPostaladdress',
        'IcemacAddressbookAddressPhonenumber',
        'IcemacAddressbookAddressEmailaddress',
        'IcemacAddressbookAddressHomepageaddress',
        'IcemacAddressbookFileFileFile',
        'IcemacAddressbookKeywordKeyword'] ==
        address_book.orders.byNamespace(ENTITIES))


def test_addressbook__create_address_book_infrastructure__4(address_book):
    """`create_address_book_infrastructure` creates only the entity order."""
    assert [ENTITIES] == list(address_book.orders.namespaces())


def test_addressbook__AddressBook__1(address_book):
    """`AddressBook` fulfills the `IAddressBook` interface."""
    zope.interface.verify.verifyObject(IAddressBook, address_book)


def test_addressbook__AddressBook____repr____1(address_book):
    """`__repr__()` renders name and title."""
    address_book.title = u'My address book'
    assert "<AddressBook u'ab' (u'My address book')>" == repr(address_book)


def test_addressbook__AddressBook____repr____2():
    """`__repr__()` does not break on missing name and title."""
    assert "<AddressBook None (None)>" == repr(AddressBook())


def test_addressbook__get_address_book__1(address_book):
    """Any object can be adated to IAddressBook to get the current one."""
    assert address_book == IAddressBook(42)
    assert address_book == IAddressBook(None)



from icemac.addressbook.i18n import _
import grokcore.component
import icemac.addressbook.entities
import icemac.addressbook.interfaces
import zope.schema.fieldproperty
import persistent
import zope.container.contained
import zope.globalrequest
import zope.interface


class PostalAddress(persistent.Persistent,
                    zope.container.contained.Contained):
    """A postal address."""

    zope.interface.implements(icemac.addressbook.interfaces.IPostalAddress)
    zope.schema.fieldproperty.createFieldProperties(
        icemac.addressbook.interfaces.IPostalAddress)


# postal address entity, default_attrib is the name of the attribute
# on IPerson which contains the default postal address.
postal_address_entity = icemac.addressbook.entities.create_entity(
    _(u'postal address'), icemac.addressbook.interfaces.IPostalAddress,
    PostalAddress, default_attrib='default_postal_address')


@zope.component.adapter(icemac.addressbook.interfaces.IPostalAddress)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def postal_address_title(address):
    """Title of a postal address."""
    title = _('none')
    values = [icemac.addressbook.interfaces.ITitle(getattr(address, x))
              for x in ('address_prefix', 'street', 'zip', 'city', 'country')
              if getattr(address, x)]
    if values:
        request = zope.globalrequest.getRequest()
        translated_values = []
        for val in values:
            if isinstance(val, zope.i18nmessageid.Message):
                val = zope.i18n.translate(val, context=request)
            translated_values.append(val)
        title = ', '.join(translated_values)
    return title


class EMailAddress(persistent.Persistent,
                   zope.container.contained.Contained):
    """An e-mail address."""

    zope.interface.implements(icemac.addressbook.interfaces.IEMailAddress)
    zope.schema.fieldproperty.createFieldProperties(
        icemac.addressbook.interfaces.IEMailAddress)


# e-mail address entity, default_attrib is the name of the attribute
# on IPerson which contains the default e-mail address.
e_mail_address_entity = icemac.addressbook.entities.create_entity(
    _(u'e-mail address'), icemac.addressbook.interfaces.IEMailAddress,
    EMailAddress, default_attrib='default_email_address')


@zope.component.adapter(icemac.addressbook.interfaces.IEMailAddress)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def email_address_title(email):
    """Title of an e-mail address."""
    title = _('none')
    if email.email:
        title = email.email
    return title


@grokcore.component.adapter(icemac.addressbook.interfaces.IPerson)
@grokcore.component.implementer(icemac.addressbook.interfaces.IEMailAddress)
def email_address_of_person(person):
    email_entity = icemac.addressbook.interfaces.IEntity(
        icemac.addressbook.interfaces.IEMailAddress)
    return getattr(person, email_entity.tagged_values['default_attrib'])


class HomePageAddress(persistent.Persistent,
                      zope.container.contained.Contained):
    """A home page address."""

    zope.interface.implements(icemac.addressbook.interfaces.IHomePageAddress)
    zope.schema.fieldproperty.createFieldProperties(
        icemac.addressbook.interfaces.IHomePageAddress)


# home page address entity, default_attrib is the name of the attribute
# on IPerson which contains the default home page address.
home_page_address_entity = icemac.addressbook.entities.create_entity(
    _(u'home page address'), icemac.addressbook.interfaces.IHomePageAddress,
    HomePageAddress, default_attrib='default_home_page_address')


@zope.component.adapter(icemac.addressbook.interfaces.IHomePageAddress)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def home_page_address_title(hp):
    """Title of a home page address."""
    title = _('none')
    if hp.url:
        title = unicode(hp.url)
    return title


class PhoneNumber(persistent.Persistent,
                  zope.container.contained.Contained):
    """A phone number."""

    zope.interface.implements(icemac.addressbook.interfaces.IPhoneNumber)
    zope.schema.fieldproperty.createFieldProperties(
        icemac.addressbook.interfaces.IPhoneNumber)


# phone number entity, default_attrib is the name of the attribute
# on IPerson which contains the default phone number.
phone_number_entity = icemac.addressbook.entities.create_entity(
    _(u'phone number'), icemac.addressbook.interfaces.IPhoneNumber,
    PhoneNumber, default_attrib='default_phone_number')


@zope.component.adapter(icemac.addressbook.interfaces.IPhoneNumber)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def phone_number_title(tel):
    """Title of a phone number."""
    title = _('none')
    if tel.number:
        title = tel.number
    return title


def normalize_phone_number(number, country_code):
    """Normalize a phone number to E.123 notation (but without spaces).

    See: http://de.wikipedia.org/wiki/E.123

    """
    digits = ''.join(x for x in number if x.isdigit())
    if digits.startswith('00'):
        # old writing of numbers abroad
        return digits.replace('00', '+', 1)
    if not country_code:
        # without country_code no further normalization is possible
        return digits
    if digits.startswith('0'):
        # normalize numbers without country code
        return digits.replace('0', country_code, 1)
    return '+' + digits


def default_attrib_name_to_entity(default_attrib_name):
    """Convert name of a default attrib to the entity where it is defined on"""
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities).getEntities(sorted=False)
    for candidate in entities:
        candidate_default_attrib = candidate.tagged_values.get(
            'default_attrib')
        if candidate_default_attrib == default_attrib_name:
            return candidate
    raise ValueError("Unknown default_attrib_name: %r" % default_attrib_name)



import zope.container.interfaces
import zope.event
import zope.lifecycleevent
import zope.traversing.api


def create_obj(class_, *args, **kw):
    """Create an object of class and fire created event."""
    obj = class_(*args)
    zope.event.notify(zope.lifecycleevent.ObjectCreatedEvent(obj))
    for attrib, value in kw.items():
        setattr(obj, attrib, value)
    return obj


def add(parent, obj):
    """Add `obj` to parent."""
    nc = zope.container.interfaces.INameChooser(parent)
    name = nc.chooseName('', obj)
    parent[name] = obj
    return name


def create_and_add(parent, class_, *args, **kw):
    """Create an instance of `class_`and add it to `parent`.

    *args ... used as arguments of the constructor.
    **kw ... set after creating the instance.
    """
    obj = create_obj(class_, *args, **kw)
    return add(parent, obj)


def delete(obj):
    """Delete an object from its parent."""
    name = zope.traversing.api.getName(obj)
    parent = zope.traversing.api.getParent(obj)
    del parent[name]


def iter_by_interface(container, interface):
    """Iterate a container and return only objects providing `interface`."""
    for obj in container.values():
        if interface.providedBy(obj):
            yield obj


def unique_by_attr_factory(attr_name, error_message):
    """Return a function checking `attr_name` is unique on parent."""
    def unique(obj, event=None):
        """Make sure `attr_name` is unique on obj's parent.

        May be used as handler for object events.

        """
        assert getattr(obj, '__parent__', None)
        container = obj.__parent__
        values = [getattr(x, attr_name) for x in container.values()]
        if len(values) != len(set(values)):
            raise zope.interface.Invalid(error_message)
    return unique


def dotted_name(cls):
    """Return the dotted name of a class."""
    return "{0.__module__}.{0.__name__}".format(cls)



import icemac.addressbook.generations.utils
import icemac.addressbook.interfaces
import zope.component


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Update sort order of entities: person should be in the second place."""
    person = icemac.addressbook.interfaces.IEntity(
        icemac.addressbook.interfaces.IPerson)
    order_storage = zope.component.getUtility(
        icemac.addressbook.interfaces.IOrderStorage)
    person_pos = order_storage.get(
        person.name, icemac.addressbook.interfaces.ENTITIES)
    if person_pos != 1:
        order_storage.up(
            person.name, icemac.addressbook.interfaces.ENTITIES, 5)



from icemac.addressbook.testing import set_modified
import zope.component
from zope.preference.interfaces import IDefaultPreferenceProvider


def test_metadata__MetadataGroup__1(address_book, FullPersonFactory, browser):
    """`MetadataGroup` shows the modification date of each entry."""
    # Change the modification dates to known values to show  that they are
    # independent.
    person = FullPersonFactory(address_book, u'Tester')
    set_modified(person, 2001, 1, 1)
    set_modified(person['PostalAddress'], 2002, 2, 2)
    set_modified(person['PhoneNumber'], 2003, 3, 3)
    set_modified(person['EMailAddress'], 2004, 4, 4)
    set_modified(person['HomePageAddress'], 2005, 5, 5)

    browser.login('visitor')
    browser.open(browser.PERSON_EDIT_URL)
    # The modification date of each entry is displayed:
    assert [
        '01/01/01 00:00',
        '02/02/02 00:00',
        '03/03/03 00:00',
        '04/04/04 00:00',
        '05/05/05 00:00'] == browser.etree.xpath(
        "//form/div/div/fieldset/fieldset/div[4]/div/span/text()")


def test_metadata__MetadataGroup__2(address_book, KeywordFactory, browser):
    """`MetadataGroup` normalizes times to time zone in prefs."""
    kw = KeywordFactory(address_book, u'foo')
    set_modified(kw, 2001, 1, 1)
    default_prefs = zope.component.getUtility(IDefaultPreferenceProvider)
    default_prefs.getDefaultPreferenceGroup('ab.timeZone').time_zone = (
        'Asia/Aden')

    browser.login('visitor')
    browser.open(browser.KEYWORD_EDIT_URL)
    assert '01/01/01 03:00' in browser.contents
    assert 'Modification Date (Asia/Aden)' in browser.contents
    assert 'Creation Date (Asia/Aden)' in browser.contents



from icemac.addressbook.utils import dotted_name
import zope.interface


def get_interfaces_flat(context):
    """Get a flat list of all interfaces provided by `context`."""
    return sorted(set([
        dotted_name(interface)
        for interface in zope.interface.implementedBy(
            context.__class__).flattened()] + [
        dotted_name(interface)
        for interface in zope.interface.directlyProvidedBy(
            context).flattened()]))


class Inspector(object):
    """Inspect the current view or object."""

    def show_context(self):
        yield {'key': 'context',
               'value': self.context}
        yield {'key': 'class of context',
               'value': self.context.__class__}
        yield {'key': 'base classes of context',
               'value': self.context.__class__.__bases__}

    def context_interfaces(self):
        return get_interfaces_flat(self.context)



import icemac.addressbook.metadata.interfaces
import zope.component
import zope.lifecycleevent
import zope.security.management
import zope.security.proxy


def set_current_princial_id(object, attribute):
    """Get the id of the current principal."""
    interaction = zope.security.management.queryInteraction()
    if interaction is None:
        return
    # Seting the first participating principal is enough for now.
    participation = interaction.participations[0]
    # Need to unwrap the object otherwise we get:
    # ForbiddenAttribute: ('__getitem__',
    #                      <BTrees.OOBTree.OOBTree object...>)
    # when trying to access the annotation key.
    unsafe_object = zope.security.proxy.getObject(object)
    metadata = icemac.addressbook.metadata.interfaces.IEditor(unsafe_object)
    setattr(metadata, attribute, unicode(participation.principal.title))


@zope.component.adapter(
    zope.annotation.interfaces.IAttributeAnnotatable,
    zope.lifecycleevent.IObjectCreatedEvent)
def CreatorAnnotator(object, event):
    """Set creator on object."""
    set_current_princial_id(object, 'creator')


def LastModifierAnnotator(object, event):
    """Set last modifier on object."""
    set_current_princial_id(object, 'modifier')



# -*- coding: utf-8 -*-
from icemac.addressbook.utils import dotted_name
import grokcore.component
import icemac.addressbook.interfaces
import persistent
import persistent.interfaces
import zc.sourcefactory.basic
import zope.container.contained
import zope.dottedname.resolve
import zope.interface.interfaces
import zope.schema
import zope.schema.fieldproperty
import zope.schema.interfaces
import zope.security.proxy
import zope.traversing.api


MAIN_ENTITIES_NAME_SUFFIXES = [
    'person.Person',
    'address.PostalAddress',
    'address.PhoneNumber',
    'address.EMailAddress',
    'address.HomePageAddress',
]


def sorted_entities(entities):
    """Return the entities sorted as defined in IEntityOrder."""
    order = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntityOrder)
    order = list(order)
    return sorted(entities, key=lambda x: order.index(x.name))


class Entities(object):
    """Predefined entities in the address book universe."""

    zope.interface.implements(icemac.addressbook.interfaces.IEntities)

    def getEntities(self, sorted=True):
        """Get an iterable of all known entities."""
        entities = zope.component.getAllUtilitiesRegisteredFor(
            icemac.addressbook.interfaces.IEntity)
        if sorted:
            entities = sorted_entities(entities)
        return entities

    def getMainEntities(self, sorted=True):
        """Get an iterable of the most important entities."""
        entities = [
            zope.component.getUtility(
                icemac.addressbook.interfaces.IEntity,
                name='icemac.addressbook.' + suffix)
            for suffix in MAIN_ENTITIES_NAME_SUFFIXES]
        if sorted:
            entities = sorted_entities(entities)
        return entities


class PersistentEntities(Entities, zope.container.btree.BTreeContainer):
    """Predefined entities and user defined fields in the address book."""


@zope.component.adapter(basestring)
@zope.interface.implementer(icemac.addressbook.interfaces.IEntity)
def entity_by_name(name):
    """Adapt Entity.name (not Entity.class_name!) to entity."""
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities).getEntities(sorted=False)
    for candidate in entities:
        if candidate.name == name:
            return candidate
    raise ValueError("Unknown name: %r" % name)


@zope.component.adapter(zope.interface.interfaces.IInterface)
@zope.interface.implementer(icemac.addressbook.interfaces.IEntity)
def entity_by_interface(interface):
    """Adapt an interface to its entity."""
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities).getEntities(sorted=False)
    for entity in entities:
        if entity.interface == interface:
            return entity
    # no entity found, create one on the fly, so all entities (even not
    # preconfigured ones) can be used the same way.
    return Entity(None, interface, None)


@zope.component.adapter(persistent.interfaces.IPersistent)
@zope.interface.implementer(icemac.addressbook.interfaces.IEntity)
def entity_by_obj(obj):
    """Adapt instance to entity."""
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities).getEntities(sorted=False)
    for candidate in entities:
        if candidate.interface.providedBy(obj):
            return candidate
    raise ValueError("Unknown obj: %r" % obj)


class EntityOrder(object):
    """Global entity order utility."""

    zope.interface.implements(icemac.addressbook.interfaces.IEntityOrder)

    @property
    def order_storage(self):
        return zope.component.getUtility(
            icemac.addressbook.interfaces.IOrderStorage)

    def _get_entity_name(self, entity):
        try:
            return entity.name
        except ValueError:
            raise KeyError('Entity %r in not known in entity order.' % entity)

    def get(self, entity):
        """Get the index of the entity in the entity order."""
        return self.order_storage.get(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES)

    def isFirst(self, entity):
        """Tell whether `entity` comes first in the entity order."""
        return self.order_storage.isFirst(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES)

    def isLast(self, entity):
        """Tell whether `entity` comes last in the entity order."""
        return self.order_storage.isLast(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES)

    def __iter__(self):
        """Iterate over the entities sorted by order."""
        return iter(self.order_storage.byNamespace(
            icemac.addressbook.interfaces.ENTITIES))

    def up(self, entity, delta=1):
        """Move the entity one position up in the entity order."""
        return self.order_storage.up(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES, delta)

    def down(self, entity, delta=1):
        """Move the entity one position down in the entity order."""
        return self.order_storage.down(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES, delta)


class ChoiceFieldValuesSource(zc.sourcefactory.basic.BasicSourceFactory):
    """Source containing the values of a choice field."""

    def __init__(self, values):
        super(ChoiceFieldValuesSource, self).__init__()
        self.values = values

    def getValues(self):
        return self.values

    def getTitle(self, value):
        return value


@grokcore.component.adapter(icemac.addressbook.interfaces.IField)
@grokcore.component.implementer(zope.schema.interfaces.IField)
def user_field_to_schema_field(field):
    """Convert a user defined field (IField) into a zope.schema field."""
    if field.type == 'Choice':
        schema_field = zope.schema.Choice(
            title=field.title, description=field.notes, required=False,
            source=ChoiceFieldValuesSource(field.values))
    else:
        schema_field = getattr(zope.schema, field.type)(
            title=field.title, description=field.notes, required=False)
    schema_field.__name__ = str(field.__name__)
    schema_field.interface = (
        icemac.addressbook.interfaces.IUserFieldStorage)
    return schema_field


def index(key, list, default):
    """Index of `key` in `list` but `default` when it is not in `list`."""
    try:
        return list.index(key)
    except ValueError:
        return default


def sorted_fields(fields, field_order):
    """Return the fields sorted by their index in the `field_order`.

    Fields which are not in the `field_order` are sorted to the end
    accordingly to their position in the `fields` list.

    """
    raw_field_order = [x[0] for x in fields]
    return sorted(
        fields,
        key=lambda (name, field): index(
            name, field_order, 100000 + raw_field_order.index(name)))


class FakeObject(object):
    """Provider for an interface for the `getAdapters` call."""


class Entity(object):
    """An entity int the address book universe."""

    # Use `create_entity` factory for easier usage.
    # Caution: This class is only a base class, in most cases you will use
    #          EditableEntity (see below).

    zope.interface.implements(icemac.addressbook.interfaces.IEntity)
    zope.schema.fieldproperty.createFieldProperties(
        icemac.addressbook.interfaces.IEntityRead)

    def __init__(self, title, interface, class_name, **kw):
        # To create an entity, use `create_entity` factory (see below) which
        # also does the ZCA set up.
        self.title = title
        self.interface = interface
        self.class_name = class_name
        # _get_raw_fields_unordered needs an instance which provides the
        # interface to look up the user defined fields which are named
        # adapters:
        self._fake_object = FakeObject()
        zope.interface.directlyProvides(self._fake_object, self.interface)
        # Additional keyword arguments are stored as tagged values
        self._tagged_values = kw

    @property
    def name(self):
        """Uniqe name of the entity which only contains letters."""
        if not self.class_name:
            raise ValueError(
                "Entity has no `class_name` set, so `name` cannot be computed."
            )
        parts = self.class_name.replace('_', '.').split('.')
        return ''.join(x.capitalize() for x in parts)

    @property
    def tagged_values(self):
        """Dict of tagged values of the entity."""
        return self._tagged_values.copy()

    @property
    def order_storage_namespace(self):
        """Get the name space used in the order storage."""
        return '%s%s' % (
            icemac.addressbook.interfaces.FIELD_NS_PREFIX, self.name)

    def getRawFields(self, sorted=True):
        """Get (name, field) tuples of the schema fields on the entity.

        When `sorted` is true the fields are sorted using the field order.

        Returns static (zope.schema) and user defined (IField) fields.

        """
        raw_fields = self._get_raw_fields_unordered()
        if not sorted:
            return raw_fields
        return sorted_fields(list(raw_fields), self.getFieldOrder())

    def getFields(self, sorted=True):
        """Get (name, field) tuples of the schema fields on the entity.

        When `sorted` is true the fields are sorted using the field order.

        Converts user defined fields (see IField) into zope.schema fields.

        """
        for name, field in self.getRawFields(sorted=sorted):
            yield name, zope.schema.interfaces.IField(field)

    def getFieldValues(self, sorted=True):
        """Get list of the schema fields on the entity.

        When `sorted` is true the fields are sorted using the field order.

        Converts user defined fields (see IField) into zope.schema fields.

        """
        return [field for name, field in self.getFields(sorted=sorted)]

    def getRawField(self, field_name):
        """Get a field by its name."""
        return dict(self.getRawFields(sorted=False))[field_name]

    def getField(self, field_name):
        """Get a zope.schema field by its name."""
        return dict(self.getFields(sorted=False))[field_name]

    def getClass(self):
        """Get the class object for `self.class_name`."""
        if self.class_name:
            return zope.dottedname.resolve.resolve(self.class_name)
        raise ValueError("class_name is not set.")

    def getFieldOrder(self):
        """Get the ordered names of the fields."""
        order_storage = zope.component.queryUtility(
            icemac.addressbook.interfaces.IOrderStorage)
        try:
            return order_storage.byNamespace(self.order_storage_namespace)
        except (KeyError, ValueError, AttributeError):
            # Either the order_storage is None or the namespace cannot be
            # computed or it is unknown, so we can't order the fields.
            return []

    # IEntityWrite

    def addField(self, field):
        """Add a user defined field to the entity."""
        # The field needs to be stored in the entities utility:
        parent = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        name = icemac.addressbook.utils.add(parent, field)
        # The field needs to be an adapter:
        sm = zope.component.hooks.getSiteManager()
        sm.registerAdapter(
            FieldAdapterFactory(field),
            provided=icemac.addressbook.interfaces.IField,
            required=(icemac.addressbook.interfaces.IEntity, self.interface),
            name=name)
        field.interface = self.interface
        return name

    def removeField(self, field):
        """Remove a user defined field from the entity."""
        sm = zope.component.hooks.getSiteManager()
        # The field no longer is allowed to be an adapter:
        sm.unregisterAdapter(
            provided=icemac.addressbook.interfaces.IField,
            required=(icemac.addressbook.interfaces.IEntity, self.interface),
            name=zope.traversing.api.getName(field))
        field.interface = None
        # The field needs to be removed from the entities utility:
        icemac.addressbook.utils.delete(field)

    def setFieldOrder(self, field_names):
        """Update the order of the fields like in `field_names`."""
        order_storage = zope.component.getUtility(
            icemac.addressbook.interfaces.IOrderStorage)
        existing_field_names = [x[0] for x in self._get_raw_fields_unordered()]
        ns = self.order_storage_namespace
        # Removing exising contents of namespace in order storage while
        # creating the namespace when it does not yet exist
        order_storage.truncate(ns)
        for name in field_names:
            if name not in existing_field_names:
                # We do not allow to write arbitrary data here:
                continue
            order_storage.add(name, ns)

    # private

    def _get_raw_fields_unordered(self):
        """Get the raw fields not ordered."""
        for name, field in zope.schema.getFieldsInOrder(self.interface):
            yield name, field
        # self._fake_object is needed here as the interfaces provided by the
        # objects are used in the look-up
        adapters = zope.component.getAdapters(
            (self, self._fake_object), icemac.addressbook.interfaces.IField)
        for name, field in adapters:
            yield str(field.__name__), field


class EditableEntity(Entity):
    """Special entity which is editable.

    This means that new fields can be added and the fields can be sorted.
    """

    zope.interface.implementsOnly(
        icemac.addressbook.interfaces.IEditableEntity)


def create_entity(title, interface, class_, **kw):
    """Factory to create an editable entity and to the ZCA set up."""
    entity = EditableEntity(title, interface, dotted_name(class_), **kw)
    zope.interface.classImplements(
        class_, icemac.addressbook.interfaces.IMayHaveUserFields)
    return entity


class Field(persistent.Persistent, zope.container.contained.Contained):
    """User defined field."""

    zope.interface.implements(icemac.addressbook.interfaces.IField)
    zope.schema.fieldproperty.createFieldProperties(
        icemac.addressbook.interfaces.IField)
    interface = None


class FieldAdapterFactory(persistent.Persistent):
    """Factory to register a field as an adapter."""

    def __init__(self, field):
        self._field = field

    def __call__(self, *args):
        return self._field


class FieldStorage(persistent.Persistent):
    """Storage for field values in annotations."""

    zope.component.adapts(
        icemac.addressbook.interfaces.IMayHaveUserFields)
    zope.interface.implements(
        icemac.addressbook.interfaces.IUserFieldStorage)

    def __getattr__(self, attrib):
        # We have no default values on the class nor we might know them.
        if not attrib.startswith('__'):
            return self.__dict__.get(attrib, None)
        # Cloning a person tries to access __reduce_ex__ and
        # __reduce__ which are not in __dict__. It is expeced that
        # they are either methods or not existing, so None is no valid
        # return value.
        raise AttributeError(attrib)


field_storage = zope.annotation.factory(
    FieldStorage, key='icemac.userfield.storage')


def get_bound_schema_field(obj, entity, field, default_attrib_fallback=True):
    """Return a bound zope.schema field for `entity` and `field` on `obj`.

    If `default_attrib_fallback` is true and `obj` does not provide the
    `entity` interface fall back to the `default_attrib`

    """
    if default_attrib_fallback and not entity.interface.providedBy(obj):
        # If the entity is for another object, we expect to find the entity
        # on a default_attrib.
        obj = getattr(obj, entity.tagged_values['default_attrib'])
    if icemac.addressbook.interfaces.IField.providedBy(field):
        # User defined fields need to be adapted. Additionally security
        # proxy must be removed as otherwise access fails. This might be a
        # security hole if access to this function is not properly
        # protected, but there is no other way.
        obj = zope.security.proxy.getObject(obj)
        obj = icemac.addressbook.interfaces.IUserFieldStorage(obj)
        field = zope.schema.interfaces.IField(field)
    return field.bind(obj)



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.addressbook.fieldsource
import icemac.addressbook.sources
import zope.interface
import zope.schema


class IPersonLists(zope.interface.Interface):
    """Person lists preferences."""

    columns = zope.schema.List(
        title=_('columns'), required=True,
        value_type=zope.schema.Choice(
            title=_('columns'),
            source=icemac.addressbook.fieldsource.source))

    order_by = zope.schema.Choice(
        title=_('order by'),
        source=icemac.addressbook.fieldsource.source)

    sort_direction = zope.schema.Choice(
        title=_('sort direction'),
        source=icemac.addressbook.sources.asc_desc_source)


class IPersonListTab(zope.interface.Interface):
    """Preferences of the person list tab."""

    batch_size = zope.schema.Int(title=_('batch size'), min=1)


class ITimeZone(zope.interface.Interface):
    """Preferred time zone."""

    time_zone = zope.schema.Choice(
        title=_('Time zone'), source=icemac.addressbook.interfaces.time_zones)



import pytest


def test_fixtures__UserFactory__1(address_book, UserFactory):
    """It raises a `LookupError` if the role is unknown."""
    with pytest.raises(LookupError) as err:
        UserFactory(address_book, u'first_name', u'last_name',
                    u'email@example.com', u'password', [u'unknown-role'])
    assert "Role title u'unknown-role' unknown." == str(err.value)



import z3c.menu.ready2go
import zope.interface


class IMasterData(z3c.menu.ready2go.ISiteMenu):
    """Menu containing viewlets which provide links to edit the master data."""


class IMasterDataMenuItemOn(zope.interface.Interface):
    """List of view names.

    For these names the master data menu item in the main navigation should be
    highlighted.
    """



from ..base import BaseForm, BaseView
from mock import Mock, patch
import pytest


def test_base__BaseView__url__1():
    """It prefixes the `view_name` with `@@`."""
    view = BaseView()
    view.request = Mock()
    with patch('zope.traversing.browser.absoluteURL') as absoluteURL:
        absoluteURL.return_value = 'http://base.url'
        assert 'http://base.url/@@index.html' == view.url(Mock(), 'index.html')


def test_base__BaseView__url__2():
    """It does not prefix `view_name` starting with `++`."""
    view = BaseView()
    view.request = Mock()
    with patch('zope.traversing.browser.absoluteURL') as absoluteURL:
        absoluteURL.return_value = 'http://base.url'
        assert 'http://base.url/++path++foo' == view.url(Mock(), '++path++foo')


def test_base__BaseView__url__3():
    """It adds keyword arguments as query arguments."""
    view = BaseView()
    view.request = Mock()
    with patch('zope.traversing.browser.absoluteURL') as absoluteURL:
        absoluteURL.return_value = 'http://base.url'
        assert 'http://base.url/@@view?a=2&b=q&b=w' == view.url(
            Mock(), 'view', a=2, b=['q', 'w'])


def test_base__BaseForm__fields__1():
    """It cannot be set twice."""
    form = BaseForm()
    form.fields = Mock()
    with pytest.raises(ValueError):
        form.fields = Mock()



# The about screen displays some information about the application, its
# version and licenses. It is accessable everywhere in the application.

ABOUT_URL = 'http://localhost/@@about.html'


def test_about__About__1(address_book, browser):
    """The about dialog is displayed on the root view."""
    from icemac.addressbook import copyright
    from pkg_resources import get_distribution
    # The root view is basic auth protected, so we have to log-in to see
    # the root view.
    browser.login('mgr')
    browser.open('http://localhost')
    # There is a link on the root view pointing to the about dialog:
    browser.getLink(id="about-view").click()
    assert ABOUT_URL == browser.url
    # The about dialog displays the version number of the address book:
    assert get_distribution('icemac.addressbook').version in browser.contents
    # ... and the copyright string (it is contained twice, as the footer
    # contains it, too):
    assert 2 == browser.contents.count(copyright)
    # There is a link to VistaIco as the license for using the icons
    # requires this:
    assert 'http://www.vistaico.com' == browser.getLink('VistaICO.com').url


def test_about__About__2(address_book, browser):
    """The about dialog is displayed on login screen."""
    # The login screen can be accessed by anonymous users:
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    # There is a link on the login view pointing to the about dialog:
    browser.getLink(id="about-view").click()
    assert ABOUT_URL == browser.url


def test_about__About__3(address_book, PersonFactory, browser):
    """The about dialog is displayed inside the application."""
    # The about screen can be accessed on every object, for example we
    # use a person:
    PersonFactory(address_book, u'Tester')
    browser.login('visitor')
    browser.open(browser.PERSON_EDIT_URL)
    browser.getLink(id="about-view").click()
    assert ABOUT_URL == browser.url



# -*- coding: latin-1 -*-
import icemac.addressbook.generations.utils


generation = 7


def evolve(context):
    """Install user defined fields utility.
    """
    icemac.addressbook.generations.utils.update_address_book_infrastructure(
        context)



import icemac.addressbook.browser.base
import icemac.addressbook.browser.menus.menu
import icemac.addressbook.browser.search.result.handler.base


class Names(icemac.addressbook.browser.search.result.handler.base.Base,
            icemac.addressbook.browser.base.BaseView):
    """Comma separed list of person names."""

    def person_list(self):
        return ', '.join(
            icemac.addressbook.interfaces.IPersonName(person).get_name()
            for person in self.persons)

    def person_count(self):
        return len(self.persons)


names = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    'person-names.html')



from icemac.addressbook.interfaces import ITitle


def test_adapter__default_title__1(zcmlS):
    """`default_title` returns a string unchanged."""
    assert 'asdf' == ITitle('asdf')


def test_adapter__default_title__2(zcmlS):
    """`default_title` returns a unicode unchanged."""
    assert u'qwe' == ITitle(u'qwe')


def test_adapter__default_title__3(zcmlS):
    """`default_title` returns a the string representation of other objects."""
    assert ITitle(object()).startswith('<object object at 0x')



"""Search by `name` index."""
from icemac.addressbook.i18n import _
import icemac.addressbook.browser.menus.menu
import icemac.addressbook.browser.search.base
import zope.catalog.interfaces
import zope.component
import zope.interface
import zope.schema


class SearchView(icemac.addressbook.browser.search.base.BaseView):
    """View which represents the name search."""

    form_explanation = _(
        'You may use wildcards in this search: Use ? for a single character '
        'or * for multiple characters.')


class ISearchFields(zope.interface.Interface):
    """Fields displayed in the search form."""

    search_term = zope.schema.TextLine(
        title=_('Name of a person'),
        description=_('You may use ? as wildcard for a single character or * '
                      'as wildcard for multiple characters.'))


class SearchForm(icemac.addressbook.browser.search.base.BaseSearchForm):

    interface = ISearchFields


class Search(icemac.addressbook.browser.search.base.BaseSearch):
    """The actual search."""

    zope.component.adapts(SearchView)

    def search(self, search_term):
        catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
        result_set = catalog.searchResults(name=search_term)
        return result_set


view = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    'name_search.html')



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.addressbook.entities
import icemac.addressbook.file.interfaces
import icemac.addressbook.interfaces
import icemac.addressbook.keyword
import icemac.addressbook.orderstorage
import icemac.addressbook.preferences.default
import zope.schema.fieldproperty
import icemac.addressbook.utils
import zc.catalog.catalogindex
import zope.app.appsetup.bootstrap
import zope.authentication.interfaces
import zope.catalog.catalog
import zope.catalog.interfaces
import zope.catalog.text
import zope.component
import zope.component.hooks
import zope.container.btree
import zope.container.interfaces
import zope.event
import zope.index.text.lexicon
import zope.interface
import zope.intid
import zope.intid.interfaces
import zope.location
import zope.location.interfaces
import zope.pluggableauth.authentication
import zope.pluggableauth.interfaces
import zope.pluggableauth.plugins.principalfolder
import zope.site.site


class AddressBook(zope.container.btree.BTreeContainer,
                  zope.site.site.SiteManagerContainer):
    """An address book."""

    zope.interface.implements(icemac.addressbook.interfaces.IAddressBook)
    zope.schema.fieldproperty.createFieldProperties(
        icemac.addressbook.interfaces.IAddressBook)

    entities = None
    importer = None
    keywords = None
    principals = None
    orders = None

    def __repr__(self):
        """Nice representation of the address book."""
        return "<AddressBook %r (%r)>" % (self.__name__, self.title)

    @property
    def time_zone(self):
        return self._timezone_preference_group.time_zone

    @time_zone.setter
    def time_zone(self, value):
        self._timezone_preference_group.time_zone = value

    # private

    @property
    def _timezone_preference_group(self):
        default_prefs = zope.component.getUtility(
            zope.preference.interfaces.IDefaultPreferenceProvider)
        return default_prefs.getDefaultPreferenceGroup('ab.timeZone')


address_book_entity = icemac.addressbook.entities.create_entity(
    _(u'address book'),
    icemac.addressbook.interfaces.IAddressBook, AddressBook)


@grok.adapter(None)
@grok.implementer(icemac.addressbook.interfaces.IAddressBook)
def get_address_book(context):
    """Get the current address book from every context."""
    return zope.component.hooks.getSite()


def create_and_register(addressbook, attrib_name, class_, interface, name=''):
    """Create an object on an attribute and register it as local utility."""
    obj = getattr(addressbook, attrib_name, None)
    if obj is not None:
        return obj
    site_mgr = addressbook.getSiteManager()
    obj = icemac.addressbook.utils.create_obj(class_)
    setattr(addressbook, attrib_name, obj)
    zope.location.locate(obj, addressbook, '++attribute++' + attrib_name)
    site_mgr.registerUtility(obj, interface, name=name)
    return obj


def add_entity_to_order(orders, iface):
    """Add an entitiy to the entity orders."""
    orders.add(icemac.addressbook.interfaces.IEntity(iface).name,
               icemac.addressbook.interfaces.ENTITIES)


@zope.component.adapter(
    icemac.addressbook.interfaces.IAddressBook,
    zope.container.interfaces.IObjectAddedEvent)
def create_address_book_infrastructure(addressbook, event=None):
    """Create the initial infrastructure ...

    ... or update the existing one to the current requirements when using a
    generation.
    """
    # add site manager
    if not zope.location.interfaces.ISite.providedBy(addressbook):
        site_manager = zope.site.site.LocalSiteManager(addressbook)
        addressbook.setSiteManager(site_manager)

    # add keywords container
    create_and_register(
        addressbook, 'keywords', icemac.addressbook.keyword.KeywordContainer,
        icemac.addressbook.interfaces.IKeywords)

    # add principals folder
    create_and_register(
        addressbook, 'principals',
        zope.pluggableauth.plugins.principalfolder.PrincipalFolder,
        zope.pluggableauth.interfaces.IAuthenticatorPlugin,
        name=u'icemac.addressbook.principals')

    # add entities utility
    create_and_register(
        addressbook, 'entities',
        icemac.addressbook.entities.PersistentEntities,
        icemac.addressbook.interfaces.IEntities)

    # add order storage utility
    orders = create_and_register(
        addressbook, 'orders',
        icemac.addressbook.orderstorage.OrderStorage,
        icemac.addressbook.interfaces.IOrderStorage)

    add_entity_to_order(orders, icemac.addressbook.interfaces.IAddressBook)
    add_entity_to_order(orders, icemac.addressbook.interfaces.IPerson)
    add_entity_to_order(orders, icemac.addressbook.interfaces.IPersonDefaults)
    add_entity_to_order(orders, icemac.addressbook.interfaces.IPostalAddress)
    add_entity_to_order(orders, icemac.addressbook.interfaces.IPhoneNumber)
    add_entity_to_order(orders, icemac.addressbook.interfaces.IEMailAddress)
    add_entity_to_order(orders, icemac.addressbook.interfaces.IHomePageAddress)
    add_entity_to_order(orders, icemac.addressbook.file.interfaces.IFile)
    add_entity_to_order(orders, icemac.addressbook.interfaces.IKeyword)

    zope.event.notify(AddressBookCreated(addressbook))


class AddressBookCreated(object):
    """Event which signals that an address book has been created.

    Subscribers can expect that the new address book is a site (but not set
    as such).
    """

    def __init__(self, address_book):
        self.address_book = address_book


@grok.subscribe(AddressBookCreated)
def add_more_addressbook_infrastructure(event):
    """Add infrastructure which depends on address book as site manager."""
    addressbook = event.address_book
    try:
        old_site = zope.component.hooks.getSite()
        zope.component.hooks.setSite(addressbook)

        # intid utility
        zope.app.appsetup.bootstrap.ensureUtility(
            addressbook, zope.intid.interfaces.IIntIds, '',
            zope.intid.IntIds)

        # catalog
        catalog = zope.app.appsetup.bootstrap.ensureUtility(
            addressbook, zope.catalog.interfaces.ICatalog, '',
            zope.catalog.catalog.Catalog)
        if catalog is None:
            catalog = zope.component.getUtility(
                zope.catalog.interfaces.ICatalog)

        # indexes
        if 'keywords' not in catalog:
            catalog['keywords'] = zc.catalog.catalogindex.SetIndex(
                'get_titles', icemac.addressbook.interfaces.IKeywordTitles,
                field_callable=True)
        if 'name' not in catalog:
            catalog['name'] = zope.catalog.text.TextIndex(
                'get_name', icemac.addressbook.interfaces.IPersonName,
                field_callable=True,
                lexicon=zope.index.text.lexicon.Lexicon(
                    zope.index.text.lexicon.Splitter(),
                    zope.index.text.lexicon.CaseNormalizer()))

        # authenticator (PAU)
        pau = zope.app.appsetup.bootstrap.ensureUtility(
            addressbook, zope.authentication.interfaces.IAuthentication,
            '',
            zope.pluggableauth.authentication.PluggableAuthentication)
        if pau is not None:
            pau.credentialsPlugins = (u'No Challenge if Authenticated',
                                      u'Flashed Session Credentials',)
            pau.authenticatorPlugins = (u'icemac.addressbook.principals',)

        # principal annotation utility
        zope.app.appsetup.bootstrap.ensureUtility(
            addressbook,
            zope.principalannotation.interfaces.IPrincipalAnnotationUtility,
            '', zope.principalannotation.utility.PrincipalAnnotationUtility)

        # default preferences
        icemac.addressbook.preferences.default.add(addressbook)
    finally:
        zope.component.hooks.setSite(old_site)



from icemac.addressbook.namechooser.interfaces import IDontReuseNames
from icemac.addressbook.namechooser.interfaces import INameSuffix
from icemac.addressbook.namechooser.namechooser import DontReuseNames
from icemac.addressbook.namechooser.namechooser import NameSuffix
from zope.interface.verify import verifyObject
import mock
import pytest
import zope.annotation.interfaces
import zope.container.sample
import zope.interface


@pytest.yield_fixture(scope='function')
def NameChooserFactory(zcmlS):
    """Get a function to create a name chooser."""
    patchers = []

    def create_name_chooser(name_suffix, patchers=patchers):
        """Create a name chooser with `name_suffix` as current suffix value."""
        nc = DontReuseNames(mock.MagicMock())
        patcher = mock.patch.object(
            DontReuseNames, 'name_suffix', new_callable=mock.PropertyMock)
        name_suffix_mock = patcher.start()
        name_suffix_mock.return_value = name_suffix
        patchers.append(patcher)  # handover the patcher to tear down
        return nc
    yield create_name_chooser
    for patcher in patchers:
        patcher.stop()


def test_namechooser__NameSuffix__1():
    """`NameSuffix` conforms to INameChooser."""
    assert verifyObject(INameSuffix, NameSuffix())


def test_namechooser__NameSuffix__2():
    """`NameSuffix` starts with a value of zero."""
    assert u'0' == unicode(NameSuffix())


def test_namechooser__NameSuffix__3():
    """`NameSuffix` sums up."""
    name_suffix = NameSuffix()
    name_suffix += 3
    name_suffix += 2
    assert u'5' == unicode(name_suffix)


def test_namechooser__DontReuseNames__chooseName__1(NameChooserFactory):
    """`chooseName()` includes a suffix even for the first name."""
    namechooser = NameChooserFactory(0)
    assert u'foo-1' == namechooser.chooseName('foo', object())


def test_namechooser__DontReuseNames__chooseName__2(NameChooserFactory):
    """`chooseName()` counts the suffix up."""
    namechooser = NameChooserFactory(1)
    assert u'foo-2' == namechooser.chooseName('foo', object())


def test_namechooser__DontReuseNames__chooseName__3(NameChooserFactory):
    """`chooseName()` ignores the name prefix when counting."""
    namechooser = NameChooserFactory(1)
    assert u'bar-2' == namechooser.chooseName('bar', object())


def test_namechooser__DontReuseNames__chooseName__4(NameChooserFactory):
    """`chooseName()` puts the suffix before the extension in the name."""
    namechooser = NameChooserFactory(6)
    assert u'baz-7.txt' == namechooser.chooseName('baz.txt', object())


def test_namechooser__DontReuseNames__chooseName__5(NameChooserFactory):
    """`chooseName()` strips leading + signs from name."""
    namechooser = NameChooserFactory(5)
    assert u'b+z+-6' == namechooser.chooseName('+++b+z+', object())


def test_namechooser__DontReuseNames__chooseName__6(NameChooserFactory):
    """`chooseName()` strips leading @ signs from name."""
    namechooser = NameChooserFactory(5)
    assert u'b@z@-6' == namechooser.chooseName('@@b@z@', object())


def test_namechooser__DontReuseNames__chooseName__7(NameChooserFactory):
    """`chooseName()` replaces slashes with minus signs."""
    namechooser = NameChooserFactory(5)
    assert u'-b-a-z--6' == namechooser.chooseName('/b/a/z/', object())


def test_namechooser__DontReuseNames__chooseName__8(NameChooserFactory):
    """`chooseName()` uses the class name if name is empty after stripping."""
    namechooser = NameChooserFactory(2)
    assert u'object-3' == namechooser.chooseName('+', object())


def test_namechooser__DontReuseNames__chooseName__9(NameChooserFactory):
    """`chooseName()` omits a name alredy used in the container."""
    nc = NameChooserFactory(2)
    with mock.patch.object(nc, 'name_in_use', side_effect=[True, False]):
        assert u'foo-4' == nc.chooseName('foo', object())


def test_namechooser__DontReuseNames__1(zcmlS):
    """`DontReuseNames` stores its data on the container."""
    container = zope.container.sample.SampleContainer()
    zope.interface.alsoProvides(container, IDontReuseNames)
    zope.interface.alsoProvides(
        container, zope.annotation.interfaces.IAttributeAnnotatable)
    nc = zope.container.interfaces.INameChooser(container)
    assert isinstance(nc, DontReuseNames)
    assert u'foo-1' == nc.chooseName('foo', object())
    nc2 = zope.container.interfaces.INameChooser(container)
    assert u'foo-2' == nc2.chooseName('foo', object())



import pytest
from mock import patch


@pytest.fixture(scope='function')
def checklist_data(address_book, PersonFactory):
    """Create the data for the `Checklist` tests."""
    return [PersonFactory(address_book, u'Vrozzek', first_name=u'Paul'),
            PersonFactory(address_book, u'Vranzz', first_name=u'Peter')]


def test_checklist__Checklist__1(checklist_data, browser):
    """It returns a list of check-boxes and names plus a count."""
    from gocept.testing.mock import Property
    browser.login('visitor')
    persons = (
        'icemac.addressbook.browser.search.result.handler.checklist.Checklist.'
        'persons')
    with patch(persons, Property()) as persons:
        persons.return_value = checklist_data
        browser.open('http://localhost/ab/@@person-checklist.html')
        assert browser.getControl('Paul Vrozzek').selected is False
        assert browser.getControl('Peter Vranzz').selected is False
        assert 'Number of persons: 2' in browser.contents



import pytest


def assert_masterdata_links(browser, username, link_texts):
    """Assert that the shown links match the expectations."""
    browser.login(username)
    browser.open(browser.MASTER_DATA_URL)
    assert link_texts == browser.etree.xpath(
        '//ul[@class="bullet"]/li/a/span/child::text()')

# There are some master data which can be edited by persons who are
# allowed to. There is a master data overview which shows all parts the
# user can see, this differs between the roles.


def test_masterdata_security__1(address_book, browser):
    """A manager sees all links on the master data page."""
    assert_masterdata_links(
        browser, 'mgr', ['Address book', 'Keywords', 'Users', 'Entities'])


@pytest.mark.parametrize('username', ['editor', 'visitor'])
def test_masterdata_security__2(address_book, browser, username):
    """The editor sees some links on the master data page"""
    assert_masterdata_links(browser, username, ['Keywords', 'Users'])



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import datetime
import icemac.addressbook.browser.base
import icemac.addressbook.interfaces
import icemac.truncatetext
import z3c.pagelet.browser
import z3c.table.batch
import z3c.table.column
import z3c.table.table
import zope.i18n


END_OF_DATE = datetime.date(datetime.MAXYEAR, 12, 31)
END_OF_DATETIME = datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59, 59)


# Columns

class BaseColumn(z3c.table.column.Column):
    """Column knowing how to handle entities and fields in the address book.

    It adapts the `item`  to the defined entity and gets the value from the
    specified field.

    """

    # CAUTION: This column needs the following fields to be set:

    entity = NotImplemented  # referenced entity as object
    field = NotImplemented  # referenced field on entity as object
    defaultValue = u''  # Value for display when there is no value.

    def getRawValue(self, item):
        """Compute the value, which can be None."""
        schema_field = icemac.addressbook.entities.get_bound_schema_field(
            item, self.entity, self.field)
        # Returning the value of the bound object as it might differ from item:
        return schema_field.get(schema_field.context)

    def getValue(self, item):
        """Compute the value, mostly ready for display."""
        obj = self.getRawValue(item)
        if obj is None:
            return self.defaultValue
        return obj

    renderCell = getValue


class DateTimeColumn(z3c.table.column.FormatterColumn,
                     BaseColumn):
    """Column which is sortable even with `None` values in it."""

    maxValue = END_OF_DATETIME

    def renderCell(self, item):
        value = self.getRawValue(item)
        if value:
            return self.getFormatter().format(value)
        return self.defaultValue

    def getSortKey(self, item):
        # We use the isoformat as sort key, so comparison does not break if
        # we mix timezone naive and timezone aware datetimes. And yes, we
        # know that this might produce some glitches in the sort order but
        # it is better than an HTTP-500 and better than trying to guess
        # timezone information.
        key = self.getRawValue(item)
        if key is None:
            # empty date fields should be sorted to the end of the list
            key = self.maxValue
        return key.isoformat()


class DateColumn(DateTimeColumn):
    """DateColumn which is able to sort even `None` values."""

    formatterCategory = 'date'
    maxValue = END_OF_DATE


class LinkColumn(icemac.addressbook.browser.base.BaseView,
                 z3c.table.column.LinkColumn):
    """LinkColumn which uses address book's URL computation method."""

    def getLinkURL(self, item):
        return self.url(item, self.linkName)


class TitleLinkColumn(LinkColumn):
    """Column containing the title of an object and a link to the object."""

    header = _(u'Name')
    weight = 2

    def getSortKey(self, item):
        return icemac.addressbook.interfaces.ITitle(item).lower()

    def getLinkContent(self, item):
        return icemac.addressbook.interfaces.ITitle(item)


class DeleteLinkColumn(LinkColumn):
    """Column containing the a link to delete the object."""

    header = _(u'')
    weight = 100
    linkContent = _(u'Delete')
    linkName = 'delete.html'


class TruncatedContentColumn(z3c.table.column.GetAttrColumn):
    """Column which truncates its content to `length` characters."""

    length = 20  # number of characters to display
    attrName = None  # attribute to access
    ellipsis = u'…'  # ellipsis sign
    defaultValue = u''  # default value when there is no value

    def getValue(self, obj):
        value = super(TruncatedContentColumn, self).getValue(obj)
        if value is None:
            return self.defaultValue
        result = icemac.truncatetext.truncate(
            value, self.length, self.ellipsis)
        return result


class SourceColumn(z3c.table.column.GetAttrColumn):
    """GetAttrColumn where attr is a value or iterable out of a source."""

    attrName = NotImplemented
    soure = NotImplemented  # source the values are taken from

    def getSortKey(self, item):
        # Sort case insensitive:
        return super(SourceColumn, self).getSortKey(item).lower()

    def getValue(self, obj):
        values = super(SourceColumn, self).getValue(obj)
        titles = [zope.i18n.translate(self.source.factory.getTitle(x),
                                      context=self.request)
                  for x in values]
        return u', '.join(sorted(titles, key=lambda x: x.lower()))


class KeywordsColumn(SourceColumn):
    """SourceColumn where attr is an iterable of keywords."""

    attrName = 'keywords'
    source = icemac.addressbook.interfaces.keyword_source


# Tables

class Table(z3c.table.table.Table):
    """Table which supports a no-rows-found message."""

    cssClassEven = u'table-even-row'
    cssClassOdd = u'table-odd-row'
    startBatchingAt = 1000000
    no_rows_message = u''  # Set at subclass.

    def renderTable(self):
        if self.rows:
            return super(Table, self).renderTable()
        return self.no_rows_message


class PageletTable(z3c.pagelet.browser.BrowserPagelet, Table):
    """Render the table in a pagelet which also has a template.

    When no template is required the `Table` class can be used.
    """

    def __init__(self, *args, **kw):
        super(PageletTable, self).__init__(*args, **kw)
        Table.__init__(self, *args, **kw)
    update = Table.update


# Batching

class NiceBatchProvider(z3c.table.batch.BatchProvider):
    """A batch provider with a nicer spacer."""

    batchSpacer = u'…'



from icemac.addressbook.i18n import _
import collections
import icemac.addressbook.browser.menus.menu
import icemac.addressbook.browser.search.base
import icemac.addressbook.sources
import zope.catalog.interfaces
import zope.component
import zope.interface
import zope.schema


class SearchView(icemac.addressbook.browser.search.base.BaseView):
    """View representing multi-keyword search."""

    form_explanation = _(
        'The keywords you want to search for have to be in the right box. '
        'Use the arrow buttons to move them.')


class SearchTermConcatenationSource(
        icemac.addressbook.sources.TitleMappingSource):

    _mapping = collections.OrderedDict(
        (('and', _(u'search-and', default=u'and')),
         ('or', _(u'search-or', default=u'or'))))


class ISearchForm(zope.interface.Interface):

    concat = zope.schema.Choice(
        title=_(u'search term concatenation'),
        source=SearchTermConcatenationSource(), default='and')

    keywords = zope.schema.Set(
        title=_('keywords'), required=False,
        value_type=zope.schema.Choice(
            title=_('keywords'),
            source=icemac.addressbook.interfaces.keyword_source))


class SearchForm(icemac.addressbook.browser.search.base.BaseSearchForm):

    interface = ISearchForm


concat_mapping = {'and': 'all_of',
                  'or': 'any_of'}


class Search(icemac.addressbook.browser.search.base.BaseSearch):
    """Search for a multiple keywords."""

    zope.component.adapts(SearchView)

    def search(self, concat, keywords):
        concat = concat_mapping[concat]
        keywords = tuple(x.title for x in keywords)
        catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
        result_set = catalog.searchResults(keywords={concat: keywords})
        return result_set


view = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    'multi_keyword.html')



ADD_MENU_CONTENTS = (
    '/descendant-or-self::ul[@id="add-menu-content"]/li/a/span/text()')


def test_menu__AddMenu__1(address_book, FullPersonFactory, browser):
    """The sort order in the add menu depends on the entity order."""
    FullPersonFactory(address_book, u'Tester')
    browser.login('mgr')
    browser.open(browser.PERSON_EDIT_URL)
    assert ['postal address', 'phone number'] == browser.etree.xpath(
        ADD_MENU_CONTENTS)[:2]

    browser.open(browser.ENTITIES_EDIT_URL)
    browser.getLink('up', index=3).click()
    assert 'Moved phone number up.' == browser.message
    browser.open(browser.PERSON_EDIT_URL)
    assert ['phone number', 'postal address'] == browser.etree.xpath(
        ADD_MENU_CONTENTS)[:2]



import icemac.addressbook.interfaces
import zope.interface


class RootFavIconURL(object):
    """URL to the FavIcon outside an address book."""

    def __call__(self):
        return icemac.addressbook.interfaces.DEFAULT_FAVICON


class AddressBookFavIconURL(object):
    """URL to the FavIcon selected in the address book."""

    def __call__(self):
        address_book = icemac.addressbook.interfaces.IAddressBook(self.context)
        return address_book.favicon


class FavIconData(object):
    """Data of a FavIcon."""

    zope.interface.implements(icemac.addressbook.interfaces.IFaviconData)

    def __init__(self, path, preview_path):
        self.path = path
        self.preview_path = preview_path

    def __call__(self, *args):
        return self


red = FavIconData('/++resource++img/favicon-red.ico',
                  '/++resource++img/favicon-red-preview.png')
green = FavIconData('/++resource++img/favicon-green.ico',
                    '/++resource++img/favicon-green-preview.png')
black = FavIconData('/++resource++img/favicon-black.ico',
                    '/++resource++img/favicon-black-preview.png')



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import zope.interface
import zope.schema
import zope.mimetype.interfaces
import icemac.addressbook.interfaces


class IFile(icemac.addressbook.interfaces.IPersonEntity):
    """A file uploaded from harddisk to addressbook."""

    name = zope.schema.TextLine(
        title=_(u'file name'), description=_(u'Name of the file.'))

    mimeType = zope.mimetype.interfaces.IContentTypeAware['mimeType']

    data = zope.schema.Bytes(title=_('file'))

    size = zope.schema.Int(
        title=_("Size"), description=_("Size in bytes"),
        readonly=True, required=True)

    def open(mode="r"):
        """Return an object providing access to the file data.

        Allowed values for `mode` are 'r' (read); 'w' (write); 'a' (append) and
        'r+' (read/write).  Other values cause `ValueError` to be raised.

        If the file is opened in read mode, an object with an API (but
        not necessarily interface) of `IFileReader` is returned; if
        opened in write mode, an object with an API of `IFileWriter` is
        returned; if in read/write, an object that implements both is
        returned.

        All readers and writers operate in 'binary' mode.

        """

    def openDetached():
        """Return file data disconnected from database connection.

        Read access only.
        """

    def replace(filename):
        """Replace with another file.

        CAUTION: The file given as argument gets consumed!
        """


class IFileContainer(zope.interface.Interface):
    """Marker interface for container which can contain files."""



import icemac.addressbook.generations.utils
import zope.catalog
import zope.component


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(ab):
    """Update indexes due to a bug in update search result handler."""
    catalog = zope.component.getUtility(zope.catalog.interfaces.ICatalog)
    catalog.updateIndexes()



from icemac.addressbook.testing import WebdriverPageObjectBase, Webdriver
import pytest


class POKeywordSearch(WebdriverPageObjectBase):
    """Webdriver page object for the keyword search page."""

    paths = [
        'SEARCH_BY_KEYWORD_URL',
    ]

    def search(self, keyword):
        self._selenium.addSelection(
            'id=form-widgets-keywords', 'label={}'.format(keyword))
        self._selenium.click('id=form-buttons-search')

    @property
    def num_results(self):
        return self._selenium.getCssCount('name=persons:list')

    @property
    def is_checked(self):
        return self._selenium.isChecked('name=persons:list')

    def hit_checkall(self):
        self._selenium.click('css=input.checkall')


Webdriver.attach(POKeywordSearch, 'keyword_search')


@pytest.mark.webdriver
def test_personlist__PersonTable__1_webdriver(search_data, webdriver):
    """The `checkall` checkbox deselects and reselects all persons."""
    keyword_search = webdriver.keyword_search
    webdriver.login('visitor', keyword_search.SEARCH_BY_KEYWORD_URL)
    keyword_search.search('family')
    # There are two lines of search results:
    assert 2 == keyword_search.num_results
    # The results are checked by default:
    assert keyword_search.is_checked
    keyword_search.hit_checkall()
    assert not keyword_search.is_checked
    # Select again:
    keyword_search.hit_checkall()
    assert keyword_search.is_checked


def test_personlist__ExportForm__1(address_book, browser):
    """The submit button of `ExportForm`' is not shown before searching."""
    browser.login('visitor')
    browser.open(browser.SEARCH_BY_KEYWORD_URL)
    assert [['form.buttons.search']] == browser.submit_control_names_all_forms


def test_personlist__ExportForm__2(address_book, KeywordFactory, browser):
    """The submit button of `ExportForm`' is not shown if nothing found."""
    KeywordFactory(address_book, u'example')
    browser.login('visitor')
    browser.keyword_search('example')
    assert [['form.buttons.search']] == browser.submit_control_names_all_forms


def test_personlist__ExportForm__3(search_data, browser):
    """The submit button of `ExportForm`' is shown if something found."""
    browser.login('visitor')
    browser.keyword_search('church')
    assert ([['form.buttons.search'], ['form.buttons.apply']] ==
            browser.submit_control_names_all_forms)


def test_personlist__ExportForm__4(search_data, browser):
    """It does not break if no handler is selected."""
    browser.login('visitor')
    browser.keyword_search('church')
    browser.getControl('Apply on selected persons').displayValue = []
    browser.getControl(name='form.buttons.apply').click()
    assert '200 Ok' == browser.headers['status']



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.personlist
import icemac.addressbook.browser.table
import zope.interface


@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IAddressBookBackground)
class PersonList(
        icemac.addressbook.browser.personlist.BasePersonList,
        icemac.addressbook.browser.table.PageletTable):
    """List persons in address book."""

    no_rows_message = _(
        u'There are no persons entered yet, click on "Add person" to create '
        u'one.')

    def __init__(self, *args, **kw):
        super(PersonList, self).__init__(*args, **kw)
        self.batchSize = self.prefs.personListTab.batch_size
        self.startBatchingAt = self.batchSize

    @property
    def values(self):
        """The values are stored on the context."""
        return self.context.values()



"""Module containing sources which are independent from interfaces.

Other sources are defined in interfaces.py to avoid circular references.

"""
from icemac.addressbook.i18n import _
import collections
import zc.sourcefactory.basic
import zope.component
import zope.component.hooks
import zope.globalrequest


class TitleMappingSource(zc.sourcefactory.basic.BasicSourceFactory):
    """Abstract base for sources using a mapping between value and title."""

    _mapping = None  # collections.OrderedDict, to be set in child class

    def getValues(self):
        return self._mapping.keys()

    def getTitle(self, value):
        return self._mapping[value]


class YesNoSource(TitleMappingSource):
    _mapping = collections.OrderedDict(
        ((True, _(u'yes')),
         (False, _(u'no'))))


yes_no_source = YesNoSource()


class AscDescSource(TitleMappingSource):
    _mapping = collections.OrderedDict(
        (('ascending', _(u'ascending (A-->Z)')),
         ('descending', _(u'descending (Z-->A)'))))


asc_desc_source = AscDescSource()


class FieldTypeSource(TitleMappingSource):
    _mapping = collections.OrderedDict(
        ((u'Bool', _(u'bool')),
         (u'Choice', _('choice')),
         (u'Date', _(u'date')),
         (u'Datetime', _(u'datetime')),
         (u'Decimal', _('decimal number')),
         (u'Int', _(u'integer number')),
         (u'Text', _(u'text area')),
         (u'TextLine', _(u'text line')),
         (u'Time', _(u'time')),
         (u'URI', _(u'URL')),
         ))


class SiteMenuSource(zc.sourcefactory.basic.BasicSourceFactory):
    """Source containing items of a z3c.menu.ready2go.ISiteMenu."""

    def __init__(self, view_class, menu_manager_class):
        self.menu_manager_class = menu_manager_class
        self.view_class = view_class

    @property
    def menu_manager(self):
        request = zope.globalrequest.getRequest()
        context = zope.component.hooks.getSite()
        view = self.view_class(context, request)
        menu_manager = self.menu_manager_class(context, request, view)
        menu_manager.update()
        return menu_manager

    def getValues(self):
        return self.menu_manager.viewlets

    def getTitle(self, value):
        return value.title

    def getToken(self, value):
        return value.url



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.addressbook.browser.metadata
import icemac.addressbook.entities
import icemac.addressbook.interfaces
import urlparse
import z3c.form.group
import z3c.formui.form
import zope.app.publication.traversers
import zope.publisher.interfaces
import zope.publisher.interfaces.http
import zope.security.proxy
import zope.traversing.browser


class FieldsTraverser(
        zope.app.publication.traversers.SimpleComponentTraverser):

    """Make fields traversable."""

    zope.interface.implementsOnly(zope.publisher.interfaces.IPublishTraverse)
    zope.component.adapts(icemac.addressbook.interfaces.IEntity,
                          zope.publisher.interfaces.http.IHTTPRequest)

    def publishTraverse(self, request, name):
        entities = zope.component.queryUtility(
            icemac.addressbook.interfaces.IEntities)
        try:
            return entities[name]
        except KeyError:
            return super(FieldsTraverser, self).publishTraverse(request, name)


def get_field_URL(entity, field, request, view=None):
    """Compute the URL to access a field."""
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities)
    url_parts = [zope.traversing.browser.absoluteURL(entities, request),
                 entity.__name__,
                 field.__name__]
    if view is not None:
        url_parts.append('@@%s' % view)
    return '/'.join(url_parts)


class MetadataForm(z3c.form.group.GroupForm, z3c.formui.form.Form):

    """Form to only render metadata."""

    id = 'standalone-metadata-form'
    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)


class List(object):

    """List fields of an entity."""

    def _values(self):
        # zope.schema fields are no content classes, so they have no
        # permissions defined
        return [zope.security.proxy.getObject(field)
                for name, field in self.context.getRawFields()]

    def fields(self):
        for field in self._values():
            omit = False
            if icemac.addressbook.interfaces.IField.providedBy(field):
                url = get_field_URL(self.context, field, self.request)
                delete_url = get_field_URL(
                    self.context, field, self.request, 'delete.html')
            else:
                url = delete_url = None
                omit = field.queryTaggedValue('omit-from-field-list', False)
            if not omit:
                yield {'title': field.title,
                       'delete-link': delete_url,
                       'edit-link': url,
                       'id': field.__name__}

    def metadata(self):
        # Entities are not persisitent. Because the sort order of the fields
        # is shown in the list, we show the metadata of of this sort order:
        os = zope.component.getUtility(
            icemac.addressbook.interfaces.IOrderStorage)
        try:
            context = os.byNamespace(self.context.order_storage_namespace)
        except KeyError:
            # Sort order not yet changed, so we have no metadata:
            return ''
        form = MetadataForm(context, self.request)
        # We cannot use form() here as this renders the layout template,
        # too, which is not needed here:
        form.update()
        return form.render()


class SaveSortorder(icemac.addressbook.browser.base.BaseView):

    """Save the field sort order as defined by user."""

    def __call__(self, f):
        self.context.setFieldOrder(f)
        self.send_flash(_('Saved sortorder.'))
        self.request.response.redirect(self.url(self.context))


class AddForm(icemac.addressbook.browser.base.BaseAddForm):

    """Add a new user defined field to an entity."""

    label = _(u'Add new field')
    class_ = icemac.addressbook.entities.Field
    interface = icemac.addressbook.interfaces.IField
    next_url = 'parent'

    def add(self, obj):
        self._name = zope.security.proxy.getObject(self.context).addField(obj)


class BaseForm(object):

    """Mix-in class redirecting back to the entity."""

    def redirect_to_next_url(self, *args):
        # redirect to the entity
        self.request.response.redirect(self.request.getURL(2))


class EditForm(BaseForm, icemac.addressbook.browser.base.GroupEditForm):

    """Edit a user defined field on an entity."""

    label = _(u'Edit field')
    interface = icemac.addressbook.interfaces.IField
    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)


class DeleteForm(BaseForm, icemac.addressbook.browser.base.BaseDeleteForm):

    """Delete a user defined field from an entity."""

    label = _(
        u'Caution: When you delete this field, possibly data will get lost. '
        u'Namely the data which was entered into this field when it was '
        u'displayed in a form of an object.')
    interface = icemac.addressbook.interfaces.IField
    field_names = ('type', 'title', 'notes')

    def _do_delete(self):
        # We need the name of the entity from the url here to
        # unregister the adapter.
        path = urlparse.urlsplit(self.request.getURL()).path
        entity_name = path.split('/')[-3]
        entity = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntity, name=entity_name)
        # XXX Without the following line removing the interface from the field
        #     (``field.interface = None`` in ``removeField``) fails with a
        #     ForbiddenAttribute error:
        field = zope.proxy.removeAllProxies(self.context)
        entity.removeField(field)
        # We have no need for a super call here as `removeField()` already
        # did the job.



from icemac.addressbook.i18n import _
import zope.interface
import zope.schema


class IEditor(zope.interface.Interface):
    """Editor properties."""

    creator = zope.schema.TextLine(
        title=_(u"creator"))

    modifier = zope.schema.TextLine(
        title=_(u"last modfier"))



import pytest
from mock import patch


@pytest.fixture(scope='function')
def names_data(address_book, PersonFactory):
    """Create the data for the `Names` tests."""
    return [PersonFactory(address_book, u'Vrozzek', first_name=u'Paul'),
            PersonFactory(address_book, u'Vranzz', first_name=u'Peter')]


def test_names__Names__1(names_data, browser):
    """`Names` returns a comma separated list of names and a count."""
    from gocept.testing.mock import Property
    browser.login('visitor')
    persons = (
        'icemac.addressbook.browser.search.result.handler.names.Names.'
        'persons')
    with patch(persons, Property()) as persons:
        persons.return_value = names_data
        browser.open('http://localhost/ab/@@person-names.html')
        assert 'Paul Vrozzek, Peter Vranzz' in browser.contents
        assert 'Number of persons: 2' in browser.contents



import grokcore.component as grok
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.menus.menu
import z3c.menu.ready2go.checker
import z3c.menu.ready2go.item
import zope.interface


class PersonListMenuItem(z3c.menu.ready2go.item.SiteMenuItem):
    """Sitem menu item for the person list."""


class PersonListSelectedChecker(
        z3c.menu.ready2go.checker.ViewNameSelectedChecker,
        grok.MultiAdapter):
    """Selected checker of person list for the site menu."""

    grok.adapts(zope.interface.Interface,
                icemac.addressbook.browser.interfaces.IAddressBookLayer,
                zope.interface.Interface,
                icemac.addressbook.browser.menus.menu.MainMenu,
                PersonListMenuItem)

    @property
    def selected(self):
        if super(PersonListSelectedChecker, self).selected:
            return True
        if self.view.__name__ == 'addPerson.html':
            return True
        if icemac.addressbook.interfaces.IPerson.providedBy(self.context):
            return True
        return False



# -*- coding: latin-1 -*-
import zope.generations.utility


generation = 4


def evolve(context):
    """Update the root folder to be a ``zope.site.folder.Folder`` instead of

    ``zope.app.folder.folder.Folder``.
    """
    root = zope.generations.utility.getRootFolder(context)
    context.connection.root()._p_changed = True
    root._p_changed = True



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import gocept.reference
import icemac.addressbook.address
import icemac.addressbook.entities
import icemac.addressbook.interfaces
import zope.annotation.interfaces
import zope.container.btree
import zope.interface
import zope.lifecycleevent


class Person(zope.container.btree.BTreeContainer):
    """A person."""

    zope.interface.implements(icemac.addressbook.interfaces.IPerson,
                              icemac.addressbook.interfaces.IPersonDefaults,
                              zope.annotation.interfaces.IAttributeAnnotatable)

    zope.schema.fieldproperty.createFieldProperties(
        icemac.addressbook.interfaces.IPerson, omit=['keywords'])

    keywords = gocept.reference.ReferenceCollection(
        'keywords', ensure_integrity=True)
    default_postal_address = gocept.reference.Reference(
        'default_postal_address', ensure_integrity=True)
    default_email_address = gocept.reference.Reference(
        'default_email_address', ensure_integrity=True)
    default_home_page_address = gocept.reference.Reference(
        'default_home_page_address', ensure_integrity=True)
    default_phone_number = gocept.reference.Reference(
        'default_phone_number', ensure_integrity=True)

    def get_name(self):
        values = [self.first_name, self.last_name]
        result = [x for x in values if x]
        return ' '.join(result)


person_entity = icemac.addressbook.entities.create_entity(
    _(u'person'), icemac.addressbook.interfaces.IPerson, Person)


@zope.component.adapter(icemac.addressbook.interfaces.IPerson)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def title(person):
    """Whole name of the person."""
    if not(person.first_name or person.last_name):
        return u'<no name>'
    if not person.first_name:
        return person.last_name
    return u'%s, %s' % (person.last_name, person.first_name)


def get_default_field(interface):
    """Find the field of a default value attribute on person."""
    field_name = icemac.addressbook.interfaces.IEntity(
        interface).tagged_values['default_attrib']
    return icemac.addressbook.interfaces.IPersonDefaults[field_name]


def sorted_person_defaults(name_field_tuples):
    """Sort the `name_field_tuples` as defined in entity order."""
    entity_order = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntityOrder)
    return sorted(
        name_field_tuples,
        key=lambda (name, field): entity_order.get(
            icemac.addressbook.address.default_attrib_name_to_entity(name)))


class PersonDefaultsEntity(icemac.addressbook.entities.Entity):
    """Entity which sorts the fields in IPersonDefaults alike entity order."""

    def getRawFields(self, sorted=True):
        name_fields = super(
            PersonDefaultsEntity, self).getRawFields(sorted=False)
        if sorted:
            return sorted_person_defaults(name_fields)
        return name_fields


person_defaults_entity = PersonDefaultsEntity(
    _(u'main adresses and numbers'),
    icemac.addressbook.interfaces.IPersonDefaults,
    'icemac.addressbook.person.PersonDefaults')


@zope.component.adapter(icemac.addressbook.interfaces.IPerson,
                        zope.lifecycleevent.IObjectCreatedEvent)
def person_created(person, event):
    """Create initial infrastructure or update existing infrastructure to
    current requirements (using generation)."""
    # Set default values for references as z3c.form accesses the
    # attributes before a value is assigned and gets an AttributeError
    # otherwise.
    if not hasattr(person, 'keywords'):
        person.keywords = set()
    for attrib in ['default_postal_address', 'default_email_address',
                   'default_home_page_address', 'default_phone_number']:
        if not hasattr(person, attrib):
            setattr(person, attrib, None)


class Keywords(object):

    zope.interface.implements(icemac.addressbook.interfaces.IKeywordTitles)
    zope.component.adapts(icemac.addressbook.interfaces.IPerson)

    def __init__(self, context):
        self.context = context

    def get_titles(self):
        return [icemac.addressbook.interfaces.ITitle(x)
                for x in self.context.keywords]



from .interfaces import ISearchMenuItemOn
import grokcore.component as grok
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.menus.menu
import z3c.menu.ready2go
import z3c.menu.ready2go.item
import z3c.menu.ready2go.manager
import zope.interface
import zope.viewlet.manager


class SearchMenuItem(z3c.menu.ready2go.item.SiteMenuItem):
    """Site menu item for the search tab."""


class SearchMenuItemSelectedChecker(
        icemac.addressbook.browser.menus.menu.SubscriberSelectedChecker):
    """Selected checker for the search menu item in the site menu."""

    grok.adapts(zope.interface.Interface,
                icemac.addressbook.browser.interfaces.IAddressBookLayer,
                zope.interface.Interface,
                icemac.addressbook.browser.menus.menu.MainMenu,
                SearchMenuItem)
    subscriber_interface = ISearchMenuItemOn


# Menu on the search page displaying the different search kinds:
SearchMenu = zope.viewlet.manager.ViewletManager(
    'left', z3c.menu.ready2go.IContextMenu,
    bases=(z3c.menu.ready2go.manager.MenuManager,))



def test_authviewlet__FlashedHTTPAuthenticationLogout__logout__1(
        address_book, UserFactory, browser):
    """It renders a message at logout."""
    UserFactory(address_book, u'Hans', u'Tester', u'tester@example.com',
                '1234567890', ['Visitor'])
    browser.formlogin('tester@example.com', '1234567890')
    browser.getLink('Logout').click()
    browser.html_redirect()
    assert (
        ['You have been logged out successfully.',
         'To log-in enter your username and password and submit the form.'] ==
        browser.message)



import icemac.addressbook.generations.utils
import zope.authentication.interfaces
import zope.component


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Register 'Flashed Session Credentials' at PAU."""
    pau = zope.component.getUtility(
        zope.authentication.interfaces.IAuthentication)
    pau.credentialsPlugins = (u'No Challenge if Authenticated',
                              u'Flashed Session Credentials',)
